from typing import Dict


CAMPAIGN_M: Dict[str, str] = {
    "subscribeM": "mutation SubscribeCampaign($id: String!, $uid: String!, $avatarId: String, $fileId: String, $name: String, $gender: Gender, $weight: String, $height: String, $birth: Int, $birthStr: String) {\n  subscribeCampaign(id: $id, uid: $uid, avatarId: $avatarId, fileId: $fileId, name: $name, gender: $gender, weight: $weight, height: $height, birth: $birth, birthStr: $birthStr) {\n    __typename\n    ...SubscribedCampaignFragment\n  }\n}\nfragment SubscribedCampaignFragment on CampaignSubscribed {\n  __typename\n  campaign {\n    __typename\n    id\n  }\n  user {\n    __typename\n    id\n  }\n  avatar {\n    __typename\n    ...FileFragment\n  }\n  profile {\n    __typename\n    ...FileFragment\n  }\n  name\n  subscribedDate\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}",
    "unsubscribeM": "mutation UnsubscribeCampaign($id : String!, $uid : String!) {\n  unsubscribeCampaign(id: $id, uid: $uid)\n}",
    "addAvatarM": "mutation AddAvatar($fileId : String!, $name : String, $listOrder : Int) {\n  addAvatar(fileId: $fileId, name: $name, listOrder: $listOrder) {\n    __typename\n    ...AvatarFragment\n  }\n}\nfragment AvatarFragment on Avatar {\n  __typename\n  id\n  name\n  file {\n    __typename\n    ...FileFragment\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}",
    "modifyAvatarM": "mutation ModifyAvatar($id: String!, $fileId: String, $name : String, $listOrder : Int) {\n  modifyAvatar(id: $id, fileId: $fileId, name: $name, listOrder: $listOrder)\n}",
    "removeAvatarM": "mutation RemoveAvatar($id: String!) {\n  removeAvatar(id: $id)\n}",
}
FCM_M: Dict[str, str] = {
    "setTokenM": "mutation setFCMToken($clientId: String!, $fcmToken: String!, $manufacturer: String, $brand: String, $model: String, $osVer: String, $userLang: String!, $timeZone: String) {\n  setFCMToken(clientId: $clientId, fcmToken: $fcmToken, terminalType: ANDROID, manufacturer: $manufacturer, brand: $brand, model: $model, osVer: $osVer, userLang: $userLang, timeZone: $timeZone, isAndroid: true)\n}",
}
MYINFO_M: Dict[str, str] = {
    "modifyCampaignUserProfileM": "mutation ModifyCampaignUserProfile($uid: String, $name: String, $avatarId: String, $profileId: String, $userName: String, $userProfileId: String, $birthday: Int, $birthStr: String, $gender: Gender, $height: String, $weight: String) {\n  modifyCampaignUserProfile(uid: $uid, name: $name, avatarId: $avatarId, profileId: $profileId, userName: $userName, userProfileId: $userProfileId, birthday: $birthday, birthStr: $birthStr, gender: $gender, height: $height, weight: $weight)\n}",
    "updateM": "mutation UpdateMyInfo($name: String, $emailAddress: String, $birthday: Int, $birthStr: String, $gender: Gender, $height: String, $weight: String, $profileId: String, $oldPassword: String, $newPassword: String) {\n  updateMyInfo(name: $name, emailAddress: $emailAddress, birthday: $birthday, birthStr: $birthStr, gender: $gender, height: $height, weight: $weight, profileId: $profileId, oldPassword: $oldPassword, newPassword: $newPassword) {\n    __typename\n    ...UserFragment\n  }\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}",
    "upsertCampaignUserProfileM": "mutation UpsertCampaignUserProfile($uid: String!, $name: String, $avatarId: String, $profileId: String, $userName: String, $userProfileId: String, $birthday: Int, $gender: Gender, $height: String, $weight: String) {\n  upsertCampaignUserProfile(uid: $uid, name: $name, avatarId: $avatarId, profileId: $profileId, userName: $userName, userProfileId: $userProfileId, birthday: $birthday, gender: $gender, height: $height, weight: $weight) {\n    __typename\n    ...CampaignUserProfilesFragment\n  }\n}\nfragment CampaignUserProfilesFragment on CampaignUserProfile {\n  __typename\n  id\n  user {\n    __typename\n    ...UserFragment\n  }\n  name\n  avatar {\n    __typename\n    ...AvatarFragment\n  }\n  profile {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}\nfragment AvatarFragment on Avatar {\n  __typename\n  id\n  name\n  file {\n    __typename\n    ...FileFragment\n  }\n}",
}
REVIEW_M: Dict[str, str] = {
    "updateUserFeedbackM": "mutation UpdateUserFeedback($uid: String!, $review: String!) {\n  updateUserFeedback(uid: $uid, review: $review) {\n    __typename\n    ...UserFeedbackFragment\n  }\n}\nfragment UserFeedbackFragment on UserReview {\n  __typename\n  id\n  selection\n  review\n  create\n  update\n}",
    "updateUserSelectionM": "mutation UpdateUserSelection($uid: String!, $selection: Int!) {\n  updateUserSelection(uid: $uid, selection: $selection) {\n    __typename\n    ...UserReviewFragment\n  }\n}\nfragment UserReviewFragment on UserReview {\n  __typename\n  id\n  selection\n  review\n  create\n  update\n}",
}
SIGN_M: Dict[str, str] = {
    "expireTokenM": "mutation ExpireToken {\n  expireToken {\n    __typename\n    token\n  }\n}",
    "issueTokenM": "mutation IssueToken($countryPhoneNumber: String!, $phoneNumber: String!, $password: String!, $userLang: String!, $timeZone: String!) {\n  issueToken(countryPhoneNumber: $countryPhoneNumber, phoneNumber: $phoneNumber, password: $password, userLang: $userLang, timeZone: $timeZone) {\n    __typename\n    id\n    token\n    issueDate\n    expireDate\n    user {\n      __typename\n      ...UserFragment\n    }\n    app {\n      __typename\n      ...AppFragment\n    }\n    valid\n    w360 {\n      __typename\n      token\n      secret\n      qid\n    }\n  }\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}\nfragment AppFragment on App {\n  __typename\n  id\n  name\n  packageName\n  apiKey\n  apiSecret\n  terminalType\n  description\n  status\n  versions {\n    __typename\n    ...VersionFragment\n  }\n  create\n  update\n}\nfragment VersionFragment on AppVersion {\n  __typename\n  id\n  version\n  requireUpdate\n  downloadUrl\n  description\n  create\n  update\n}",
    "signUpM": "mutation SignUp($countryPhoneCode: String!, $phoneNumber: String!, $password: String!, $confirmCode: String!, $name: String, $emailAddress: String, $profileId: String, $userLang: String!, $timeZone: String) {\n  signUp(countryPhoneCode: $countryPhoneCode, phoneNumber: $phoneNumber, password: $password, confirmCode: $confirmCode, name: $name, emailAddress: $emailAddress, profileId: $profileId, userLang: $userLang, timeZone: $timeZone) {\n    __typename\n    ...UserFragment\n  }\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}",
    "connect360M": "mutation connect360 {\n  connect360 {\n    __typename\n    id\n    token\n    issueDate\n    expireDate\n    user {\n      __typename\n      id\n      userId\n      name\n      nickname\n      gender\n      birth\n      birthStr\n      weight\n      height\n      firstName\n      lastName\n      countryCode\n      city\n      address1\n      address2\n      zipCode\n      emailAddress\n      countryPhoneCode\n      phoneNumber\n      mobilePhoneNumber\n      faxNumber\n      emailConfirm\n      description\n      status\n      file {\n        __typename\n        id\n        name\n        origName\n        contentType\n        orig {\n          __typename\n          path\n          urlPath\n          urlPathS3\n          size\n          width\n          height\n        }\n        thumbnail {\n          __typename\n          path\n          urlPath\n          urlPathS3\n          size\n          width\n          height\n        }\n        description\n        ipaddress\n        extra\n        create\n        update\n      }\n      lastLoginDate\n      changePasswordDate\n      loginFailCount\n      extra\n      xcoin\n      currentStep\n      totalStep\n      create\n      update\n      roles {\n        __typename\n        id\n        name\n        type\n        defaultRole\n        description\n        status\n        menus {\n          __typename\n          id\n          menuId\n          name\n          status\n          url\n          system\n          listOrder\n          create\n          update\n        }\n        create\n        update\n      }\n      children {\n        __typename\n        id\n        guardian {\n          __typename\n          id\n          userId\n          name\n          nickname\n          gender\n          birth\n          birthStr\n          weight\n          height\n          firstName\n          lastName\n          countryCode\n          city\n          address1\n          address2\n          zipCode\n          emailAddress\n          countryPhoneCode\n          phoneNumber\n          mobilePhoneNumber\n          faxNumber\n          emailConfirm\n          description\n          status\n          file {\n            __typename\n            id\n            name\n            origName\n            contentType\n            description\n            ipaddress\n            extra\n            create\n            update\n          }\n          lastLoginDate\n          changePasswordDate\n          loginFailCount\n          extra\n          xcoin\n          currentStep\n          totalStep\n          create\n          update\n          roles {\n            __typename\n            id\n            name\n            type\n            defaultRole\n            description\n            status\n            create\n            update\n          }\n          children {\n            __typename\n            id\n            flag\n            approval\n            listOrder\n            create\n          }\n          contacts {\n            __typename\n            id\n            name\n            phoneNumber\n            extra\n            listOrder\n            create\n            update\n          }\n        }\n        ward {\n          __typename\n          id\n          userId\n          name\n          nickname\n          gender\n          birth\n          birthStr\n          weight\n          height\n          firstName\n          lastName\n          countryCode\n          city\n          address1\n          address2\n          zipCode\n          emailAddress\n          countryPhoneCode\n          phoneNumber\n          mobilePhoneNumber\n          faxNumber\n          emailConfirm\n          description\n          status\n          file {\n            __typename\n            id\n            name\n            origName\n            contentType\n            description\n            ipaddress\n            extra\n            create\n            update\n          }\n          lastLoginDate\n          changePasswordDate\n          loginFailCount\n          extra\n          xcoin\n          currentStep\n          totalStep\n          create\n          update\n          roles {\n            __typename\n            id\n            name\n            type\n            defaultRole\n            description\n            status\n            create\n            update\n          }\n          children {\n            __typename\n            id\n            flag\n            approval\n            listOrder\n            create\n          }\n          contacts {\n            __typename\n            id\n            name\n            phoneNumber\n            extra\n            listOrder\n            create\n            update\n          }\n        }\n        flag\n        approval\n        listOrder\n        create\n      }\n      contacts {\n        __typename\n        id\n        me {\n          __typename\n          id\n          userId\n          name\n          nickname\n          gender\n          birth\n          birthStr\n          weight\n          height\n          firstName\n          lastName\n          countryCode\n          city\n          address1\n          address2\n          zipCode\n          emailAddress\n          countryPhoneCode\n          phoneNumber\n          mobilePhoneNumber\n          faxNumber\n          emailConfirm\n          description\n          status\n          file {\n            __typename\n            id\n            name\n            origName\n            contentType\n            description\n            ipaddress\n            extra\n            create\n            update\n          }\n          lastLoginDate\n          changePasswordDate\n          loginFailCount\n          extra\n          xcoin\n          currentStep\n          totalStep\n          create\n          update\n          roles {\n            __typename\n            id\n            name\n            type\n            defaultRole\n            description\n            status\n            create\n            update\n          }\n          children {\n            __typename\n            id\n            flag\n            approval\n            listOrder\n            create\n          }\n          contacts {\n            __typename\n            id\n            name\n            phoneNumber\n            extra\n            listOrder\n            create\n            update\n          }\n        }\n        name\n        phoneNumber\n        contacter {\n          __typename\n          id\n          userId\n          name\n          nickname\n          gender\n          birth\n          birthStr\n          weight\n          height\n          firstName\n          lastName\n          countryCode\n          city\n          address1\n          address2\n          zipCode\n          emailAddress\n          countryPhoneCode\n          phoneNumber\n          mobilePhoneNumber\n          faxNumber\n          emailConfirm\n          description\n          status\n          file {\n            __typename\n            id\n            name\n            origName\n            contentType\n            description\n            ipaddress\n            extra\n            create\n            update\n          }\n          lastLoginDate\n          changePasswordDate\n          loginFailCount\n          extra\n          xcoin\n          currentStep\n          totalStep\n          create\n          update\n          roles {\n            __typename\n            id\n            name\n            type\n            defaultRole\n            description\n            status\n            create\n            update\n          }\n          children {\n            __typename\n            id\n            flag\n            approval\n            listOrder\n            create\n          }\n          contacts {\n            __typename\n            id\n            name\n            phoneNumber\n            extra\n            listOrder\n            create\n            update\n          }\n        }\n        extra\n        listOrder\n        file {\n          __typename\n          id\n          name\n          origName\n          contentType\n          description\n          ipaddress\n          extra\n          create\n          update\n        }\n        create\n        update\n      }\n    }\n    app {\n      __typename\n      id\n      name\n      packageName\n      apiKey\n      apiSecret\n      terminalType\n      description\n      status\n      file {\n        __typename\n        id\n        name\n        origName\n        contentType\n        description\n        ipaddress\n        extra\n        create\n        update\n      }\n      versions {\n        __typename\n        id\n        version\n        requireUpdate\n        downloadUrl\n        description\n        create\n        update\n      }\n      create\n      update\n    }\n    w360 {\n      __typename\n      token\n      secret\n      qid\n    }\n  }\n}",
    "signInWithEmailOrPhoneM": "mutation signInWithEmailOrPhone($countryPhoneNumber: String, $phoneNumber: String, $password: String!, $emailAddress: String, $client: ClientType!, $userLang: String!, $timeZone: String!) {\n  signInWithEmailOrPhone(countryPhoneNumber: $countryPhoneNumber, phoneNumber: $phoneNumber, password: $password, emailAddress: $emailAddress, client: $client, userLang: $userLang, timeZone: $timeZone) {\n    __typename\n    id\n    token\n    issueDate\n    expireDate\n    user {\n      __typename\n      id\n      userId\n      name\n      nickname\n      gender\n      birth\n      birthStr\n      weight\n      height\n      firstName\n      lastName\n      countryCode\n      city\n      address1\n      address2\n      zipCode\n      emailAddress\n      countryPhoneCode\n      phoneNumber\n      mobilePhoneNumber\n      faxNumber\n      emailConfirm\n      description\n      status\n      file {\n        __typename\n        id\n        name\n        origName\n        contentType\n        orig {\n          __typename\n          path\n          urlPath\n          urlPathS3\n          size\n          width\n          height\n        }\n        thumbnail {\n          __typename\n          path\n          urlPath\n          urlPathS3\n          size\n          width\n          height\n        }\n        description\n        ipaddress\n        extra\n        create\n        update\n      }\n      lastLoginDate\n      changePasswordDate\n      loginFailCount\n      extra\n      xcoin\n      currentStep\n      totalStep\n      create\n      update\n      roles {\n        __typename\n        id\n        name\n        type\n        defaultRole\n        description\n        status\n        menus {\n          __typename\n          id\n          menuId\n          name\n          status\n          url\n          system\n          listOrder\n          create\n          update\n        }\n        create\n        update\n      }\n      children {\n        __typename\n        id\n        guardian {\n          __typename\n          id\n          userId\n          name\n          nickname\n          gender\n          birth\n          birthStr\n          weight\n          height\n          firstName\n          lastName\n          countryCode\n          city\n          address1\n          address2\n          zipCode\n          emailAddress\n          countryPhoneCode\n          phoneNumber\n          mobilePhoneNumber\n          faxNumber\n          emailConfirm\n          description\n          status\n          file {\n            __typename\n            id\n            name\n            origName\n            contentType\n            description\n            ipaddress\n            extra\n            create\n            update\n          }\n          lastLoginDate\n          changePasswordDate\n          loginFailCount\n          extra\n          xcoin\n          currentStep\n          totalStep\n          create\n          update\n          roles {\n            __typename\n            id\n            name\n            type\n            defaultRole\n            description\n            status\n            create\n            update\n          }\n          children {\n            __typename\n            id\n            flag\n            approval\n            listOrder\n            create\n          }\n          contacts {\n            __typename\n            id\n            name\n            phoneNumber\n            extra\n            listOrder\n            create\n            update\n          }\n        }\n        ward {\n          __typename\n          id\n          userId\n          name\n          nickname\n          gender\n          birth\n          birthStr\n          weight\n          height\n          firstName\n          lastName\n          countryCode\n          city\n          address1\n          address2\n          zipCode\n          emailAddress\n          countryPhoneCode\n          phoneNumber\n          mobilePhoneNumber\n          faxNumber\n          emailConfirm\n          description\n          status\n          file {\n            __typename\n            id\n            name\n            origName\n            contentType\n            description\n            ipaddress\n            extra\n            create\n            update\n          }\n          lastLoginDate\n          changePasswordDate\n          loginFailCount\n          extra\n          xcoin\n          currentStep\n          totalStep\n          create\n          update\n          roles {\n            __typename\n            id\n            name\n            type\n            defaultRole\n            description\n            status\n            create\n            update\n          }\n          children {\n            __typename\n            id\n            flag\n            approval\n            listOrder\n            create\n          }\n          contacts {\n            __typename\n            id\n            name\n            phoneNumber\n            extra\n            listOrder\n            create\n            update\n          }\n        }\n        flag\n        approval\n        listOrder\n        create\n      }\n      contacts {\n        __typename\n        id\n        me {\n          __typename\n          id\n          userId\n          name\n          nickname\n          gender\n          birth\n          birthStr\n          weight\n          height\n          firstName\n          lastName\n          countryCode\n          city\n          address1\n          address2\n          zipCode\n          emailAddress\n          countryPhoneCode\n          phoneNumber\n          mobilePhoneNumber\n          faxNumber\n          emailConfirm\n          description\n          status\n          file {\n            __typename\n            id\n            name\n            origName\n            contentType\n            description\n            ipaddress\n            extra\n            create\n            update\n          }\n          lastLoginDate\n          changePasswordDate\n          loginFailCount\n          extra\n          xcoin\n          currentStep\n          totalStep\n          create\n          update\n          roles {\n            __typename\n            id\n            name\n            type\n            defaultRole\n            description\n            status\n            create\n            update\n          }\n          children {\n            __typename\n            id\n            flag\n            approval\n            listOrder\n            create\n          }\n          contacts {\n            __typename\n            id\n            name\n            phoneNumber\n            extra\n            listOrder\n            create\n            update\n          }\n        }\n        name\n        phoneNumber\n        contacter {\n          __typename\n          id\n          userId\n          name\n          nickname\n          gender\n          birth\n          birthStr\n          weight\n          height\n          firstName\n          lastName\n          countryCode\n          city\n          address1\n          address2\n          zipCode\n          emailAddress\n          countryPhoneCode\n          phoneNumber\n          mobilePhoneNumber\n          faxNumber\n          emailConfirm\n          description\n          status\n          file {\n            __typename\n            id\n            name\n            origName\n            contentType\n            description\n            ipaddress\n            extra\n            create\n            update\n          }\n          lastLoginDate\n          changePasswordDate\n          loginFailCount\n          extra\n          xcoin\n          currentStep\n          totalStep\n          create\n          update\n          roles {\n            __typename\n            id\n            name\n            type\n            defaultRole\n            description\n            status\n            create\n            update\n          }\n          children {\n            __typename\n            id\n            flag\n            approval\n            listOrder\n            create\n          }\n          contacts {\n            __typename\n            id\n            name\n            phoneNumber\n            extra\n            listOrder\n            create\n            update\n          }\n        }\n        extra\n        listOrder\n        file {\n          __typename\n          id\n          name\n          origName\n          contentType\n          description\n          ipaddress\n          extra\n          create\n          update\n        }\n        create\n        update\n      }\n    }\n    app {\n      __typename\n      id\n      name\n      packageName\n      apiKey\n      apiSecret\n      terminalType\n      description\n      status\n      file {\n        __typename\n        id\n        name\n        origName\n        contentType\n        description\n        ipaddress\n        extra\n        create\n        update\n      }\n      versions {\n        __typename\n        id\n        version\n        requireUpdate\n        downloadUrl\n        description\n        create\n        update\n      }\n      create\n      update\n    }\n    valid\n    w360 {\n      __typename\n      token\n      secret\n      qid\n    }\n  }\n}",
    "signUpWithEmailAndPhoneM": "mutation signUpWithEmailAndPhone($countryPhoneCode: String!, $phoneNumber: String!, $password: String!, $name: String!, $emailAddress: String!, $phoneVerificationCode: String!, $emailVerificationCode: String!, $verificationToken: String!) {\n  signUpWithEmailAndPhone(countryPhoneCode: $countryPhoneCode, phoneNumber: $phoneNumber, password: $password, name: $name, emailAddress: $emailAddress, phoneVerificationCode: $phoneVerificationCode, emailVerificationCode: $emailVerificationCode, verificationToken: $verificationToken) {\n    __typename\n    id\n    userId\n    name\n    nickname\n    gender\n    birth\n    birthStr\n    weight\n    height\n    firstName\n    lastName\n    countryCode\n    city\n    address1\n    address2\n    zipCode\n    emailAddress\n    countryPhoneCode\n    phoneNumber\n    mobilePhoneNumber\n    faxNumber\n    emailConfirm\n    description\n    status\n    file {\n      __typename\n      id\n      name\n      origName\n      contentType\n      orig {\n        __typename\n        path\n        urlPath\n        urlPathS3\n        size\n        width\n        height\n      }\n      thumbnail {\n        __typename\n        path\n        urlPath\n        urlPathS3\n        size\n        width\n        height\n      }\n      description\n      ipaddress\n      extra\n      create\n      update\n    }\n    lastLoginDate\n    changePasswordDate\n    loginFailCount\n    extra\n    xcoin\n    currentStep\n    totalStep\n    create\n    update\n    roles {\n      __typename\n      id\n      name\n      type\n      defaultRole\n      description\n      status\n      menus {\n        __typename\n        id\n        menuId\n        name\n        status\n        url\n        system\n        listOrder\n        create\n        update\n      }\n      create\n      update\n    }\n    children {\n      __typename\n      id\n      guardian {\n        __typename\n        id\n        userId\n        name\n        nickname\n        gender\n        birth\n        birthStr\n        weight\n        height\n        firstName\n        lastName\n        countryCode\n        city\n        address1\n        address2\n        zipCode\n        emailAddress\n        countryPhoneCode\n        phoneNumber\n        mobilePhoneNumber\n        faxNumber\n        emailConfirm\n        description\n        status\n        file {\n          __typename\n          id\n          name\n          origName\n          contentType\n          description\n          ipaddress\n          extra\n          create\n          update\n        }\n        lastLoginDate\n        changePasswordDate\n        loginFailCount\n        extra\n        xcoin\n        currentStep\n        totalStep\n        create\n        update\n        roles {\n          __typename\n          id\n          name\n          type\n          defaultRole\n          description\n          status\n          create\n          update\n        }\n        children {\n          __typename\n          id\n          ward {\n            __typename\n            id\n            userId\n            name\n            nickname\n            gender\n            birth\n            birthStr\n            weight\n            height\n            firstName\n            lastName\n            countryCode\n            city\n            address1\n            address2\n            zipCode\n            emailAddress\n            countryPhoneCode\n            phoneNumber\n            mobilePhoneNumber\n            faxNumber\n            emailConfirm\n            description\n            status\n            lastLoginDate\n            changePasswordDate\n            loginFailCount\n            extra\n            xcoin\n            currentStep\n            totalStep\n            create\n            update\n          }\n          flag\n          approval\n          listOrder\n          create\n        }\n        contacts {\n          __typename\n          id\n          me {\n            __typename\n            id\n            userId\n            name\n            nickname\n            gender\n            birth\n            birthStr\n            weight\n            height\n            firstName\n            lastName\n            countryCode\n            city\n            address1\n            address2\n            zipCode\n            emailAddress\n            countryPhoneCode\n            phoneNumber\n            mobilePhoneNumber\n            faxNumber\n            emailConfirm\n            description\n            status\n            lastLoginDate\n            changePasswordDate\n            loginFailCount\n            extra\n            xcoin\n            currentStep\n            totalStep\n            create\n            update\n          }\n          name\n          phoneNumber\n          contacter {\n            __typename\n            id\n            userId\n            name\n            nickname\n            gender\n            birth\n            birthStr\n            weight\n            height\n            firstName\n            lastName\n            countryCode\n            city\n            address1\n            address2\n            zipCode\n            emailAddress\n            countryPhoneCode\n            phoneNumber\n            mobilePhoneNumber\n            faxNumber\n            emailConfirm\n            description\n            status\n            lastLoginDate\n            changePasswordDate\n            loginFailCount\n            extra\n            xcoin\n            currentStep\n            totalStep\n            create\n            update\n          }\n          extra\n          listOrder\n          file {\n            __typename\n            id\n            name\n            origName\n            contentType\n            description\n            ipaddress\n            extra\n            create\n            update\n          }\n          create\n          update\n        }\n      }\n      flag\n      approval\n      listOrder\n      create\n    }\n    contacts {\n      __typename\n      id\n      name\n      phoneNumber\n      extra\n      listOrder\n      create\n      update\n    }\n  }\n}",
    "signUpWithEmailM": "mutation SignUpWithEmail($countryPhoneCode: String!, $phoneNumber: String!, $password: String!, $name: String!, $emailAddress: String!, $emailVerificationCode: String!, $verificationToken: String!) {\n  signUpWithEmail(countryPhoneCode: $countryPhoneCode, phoneNumber: $phoneNumber, password: $password, name: $name, emailAddress: $emailAddress, emailVerificationCode: $emailVerificationCode, verificationToken: $verificationToken) {\n    __typename\n    id\n    userId\n    name\n    nickname\n    gender\n    birth\n    birthStr\n    weight\n    height\n    firstName\n    lastName\n    countryCode\n    city\n    address1\n    address2\n    zipCode\n    emailAddress\n    countryPhoneCode\n    phoneNumber\n    mobilePhoneNumber\n    faxNumber\n    emailConfirm\n    description\n    status\n    file {\n      __typename\n      id\n      name\n      origName\n      contentType\n      orig {\n        __typename\n        path\n        urlPath\n        urlPathS3\n        size\n        width\n        height\n      }\n      thumbnail {\n        __typename\n        path\n        urlPath\n        urlPathS3\n        size\n        width\n        height\n      }\n      description\n      ipaddress\n      extra\n      create\n      update\n    }\n    lastLoginDate\n    changePasswordDate\n    loginFailCount\n    extra\n    xcoin\n    currentStep\n    totalStep\n    create\n    update\n    roles {\n      __typename\n      id\n      name\n      type\n      defaultRole\n      description\n      status\n      menus {\n        __typename\n        id\n        menuId\n        name\n        status\n        url\n        system\n        listOrder\n        create\n        update\n      }\n      create\n      update\n    }\n    children {\n      __typename\n      id\n      guardian {\n        __typename\n        id\n        userId\n        name\n        nickname\n        gender\n        birth\n        birthStr\n        weight\n        height\n        firstName\n        lastName\n        countryCode\n        city\n        address1\n        address2\n        zipCode\n        emailAddress\n        countryPhoneCode\n        phoneNumber\n        mobilePhoneNumber\n        faxNumber\n        emailConfirm\n        description\n        status\n        file {\n          __typename\n          id\n          name\n          origName\n          contentType\n          orig {\n            __typename\n            path\n            urlPath\n            urlPathS3\n            size\n            width\n            height\n          }\n          thumbnail {\n            __typename\n            path\n            urlPath\n            urlPathS3\n            size\n            width\n            height\n          }\n          description\n          ipaddress\n          extra\n          create\n          update\n        }\n        lastLoginDate\n        changePasswordDate\n        loginFailCount\n        extra\n        xcoin\n        currentStep\n        totalStep\n        create\n        update\n        roles {\n          __typename\n          id\n          name\n          type\n          defaultRole\n          description\n          status\n          menus {\n            __typename\n            id\n            menuId\n            name\n            status\n            url\n            system\n            listOrder\n            create\n            update\n          }\n          create\n          update\n        }\n        children {\n          __typename\n          id\n          ward {\n            __typename\n            id\n            userId\n            name\n            nickname\n            gender\n            birth\n            birthStr\n            weight\n            height\n            firstName\n            lastName\n            countryCode\n            city\n            address1\n            address2\n            zipCode\n            emailAddress\n            countryPhoneCode\n            phoneNumber\n            mobilePhoneNumber\n            faxNumber\n            emailConfirm\n            description\n            status\n            lastLoginDate\n            changePasswordDate\n            loginFailCount\n            extra\n            xcoin\n            currentStep\n            totalStep\n            create\n            update\n          }\n          flag\n          approval\n          listOrder\n          create\n        }\n        contacts {\n          __typename\n          id\n          me {\n            __typename\n            id\n            userId\n            name\n            nickname\n            gender\n            birth\n            birthStr\n            weight\n            height\n            firstName\n            lastName\n            countryCode\n            city\n            address1\n            address2\n            zipCode\n            emailAddress\n            countryPhoneCode\n            phoneNumber\n            mobilePhoneNumber\n            faxNumber\n            emailConfirm\n            description\n            status\n            lastLoginDate\n            changePasswordDate\n            loginFailCount\n            extra\n            xcoin\n            currentStep\n            totalStep\n            create\n            update\n          }\n          name\n          phoneNumber\n          contacter {\n            __typename\n            id\n            userId\n            name\n            nickname\n            gender\n            birth\n            birthStr\n            weight\n            height\n            firstName\n            lastName\n            countryCode\n            city\n            address1\n            address2\n            zipCode\n            emailAddress\n            countryPhoneCode\n            phoneNumber\n            mobilePhoneNumber\n            faxNumber\n            emailConfirm\n            description\n            status\n            lastLoginDate\n            changePasswordDate\n            loginFailCount\n            extra\n            xcoin\n            currentStep\n            totalStep\n            create\n            update\n          }\n          extra\n          listOrder\n          file {\n            __typename\n            id\n            name\n            origName\n            contentType\n            description\n            ipaddress\n            extra\n            create\n            update\n          }\n          create\n          update\n        }\n      }\n      ward {\n        __typename\n        id\n        userId\n        name\n        nickname\n        gender\n        birth\n        birthStr\n        weight\n        height\n        firstName\n        lastName\n        countryCode\n        city\n        address1\n        address2\n        zipCode\n        emailAddress\n        countryPhoneCode\n        phoneNumber\n        mobilePhoneNumber\n        faxNumber\n        emailConfirm\n        description\n        status\n        file {\n          __typename\n          id\n          name\n          origName\n          contentType\n          orig {\n            __typename\n            path\n            urlPath\n            urlPathS3\n            size\n            width\n            height\n          }\n          thumbnail {\n            __typename\n            path\n            urlPath\n            urlPathS3\n            size\n            width\n            height\n          }\n          description\n          ipaddress\n          extra\n          create\n          update\n        }\n        lastLoginDate\n        changePasswordDate\n        loginFailCount\n        extra\n        xcoin\n        currentStep\n        totalStep\n        create\n        update\n        roles {\n          __typename\n          id\n          name\n          type\n          defaultRole\n          description\n          status\n          menus {\n            __typename\n            id\n            menuId\n            name\n            status\n            url\n            system\n            listOrder\n            create\n            update\n          }\n          create\n          update\n        }\n        children {\n          __typename\n          id\n          guardian {\n            __typename\n            id\n            userId\n            name\n            nickname\n            gender\n            birth\n            birthStr\n            weight\n            height\n            firstName\n            lastName\n            countryCode\n            city\n            address1\n            address2\n            zipCode\n            emailAddress\n            countryPhoneCode\n            phoneNumber\n            mobilePhoneNumber\n            faxNumber\n            emailConfirm\n            description\n            status\n            lastLoginDate\n            changePasswordDate\n            loginFailCount\n            extra\n            xcoin\n            currentStep\n            totalStep\n            create\n            update\n          }\n          flag\n          approval\n          listOrder\n          create\n        }\n        contacts {\n          __typename\n          id\n          me {\n            __typename\n            id\n            userId\n            name\n            nickname\n            gender\n            birth\n            birthStr\n            weight\n            height\n            firstName\n            lastName\n            countryCode\n            city\n            address1\n            address2\n            zipCode\n            emailAddress\n            countryPhoneCode\n            phoneNumber\n            mobilePhoneNumber\n            faxNumber\n            emailConfirm\n            description\n            status\n            lastLoginDate\n            changePasswordDate\n            loginFailCount\n            extra\n            xcoin\n            currentStep\n            totalStep\n            create\n            update\n          }\n          name\n          phoneNumber\n          contacter {\n            __typename\n            id\n            userId\n            name\n            nickname\n            gender\n            birth\n            birthStr\n            weight\n            height\n            firstName\n            lastName\n            countryCode\n            city\n            address1\n            address2\n            zipCode\n            emailAddress\n            countryPhoneCode\n            phoneNumber\n            mobilePhoneNumber\n            faxNumber\n            emailConfirm\n            description\n            status\n            lastLoginDate\n            changePasswordDate\n            loginFailCount\n            extra\n            xcoin\n            currentStep\n            totalStep\n            create\n            update\n          }\n          extra\n          listOrder\n          file {\n            __typename\n            id\n            name\n            origName\n            contentType\n            description\n            ipaddress\n            extra\n            create\n            update\n          }\n          create\n          update\n        }\n      }\n      flag\n      approval\n      listOrder\n      create\n    }\n    contacts {\n      __typename\n      id\n      me {\n        __typename\n        id\n        userId\n        name\n        nickname\n        gender\n        birth\n        birthStr\n        weight\n        height\n        firstName\n        lastName\n        countryCode\n        city\n        address1\n        address2\n        zipCode\n        emailAddress\n        countryPhoneCode\n        phoneNumber\n        mobilePhoneNumber\n        faxNumber\n        emailConfirm\n        description\n        status\n        file {\n          __typename\n          id\n          name\n          origName\n          contentType\n          orig {\n            __typename\n            path\n            urlPath\n            urlPathS3\n            size\n            width\n            height\n          }\n          thumbnail {\n            __typename\n            path\n            urlPath\n            urlPathS3\n            size\n            width\n            height\n          }\n          description\n          ipaddress\n          extra\n          create\n          update\n        }\n        lastLoginDate\n        changePasswordDate\n        loginFailCount\n        extra\n        xcoin\n        currentStep\n        totalStep\n        create\n        update\n        roles {\n          __typename\n          id\n          name\n          type\n          defaultRole\n          description\n          status\n          menus {\n            __typename\n            id\n            menuId\n            name\n            status\n            url\n            system\n            listOrder\n            create\n            update\n          }\n          create\n          update\n        }\n        children {\n          __typename\n          id\n          guardian {\n            __typename\n            id\n            userId\n            name\n            nickname\n            gender\n            birth\n            birthStr\n            weight\n            height\n            firstName\n            lastName\n            countryCode\n            city\n            address1\n            address2\n            zipCode\n            emailAddress\n            countryPhoneCode\n            phoneNumber\n            mobilePhoneNumber\n            faxNumber\n            emailConfirm\n            description\n            status\n            lastLoginDate\n            changePasswordDate\n            loginFailCount\n            extra\n            xcoin\n            currentStep\n            totalStep\n            create\n            update\n          }\n          ward {\n            __typename\n            id\n            userId\n            name\n            nickname\n            gender\n            birth\n            birthStr\n            weight\n            height\n            firstName\n            lastName\n            countryCode\n            city\n            address1\n            address2\n            zipCode\n            emailAddress\n            countryPhoneCode\n            phoneNumber\n            mobilePhoneNumber\n            faxNumber\n            emailConfirm\n            description\n            status\n            lastLoginDate\n            changePasswordDate\n            loginFailCount\n            extra\n            xcoin\n            currentStep\n            totalStep\n            create\n            update\n          }\n          flag\n          approval\n          listOrder\n          create\n        }\n        contacts {\n          __typename\n          id\n          name\n          phoneNumber\n          contacter {\n            __typename\n            id\n            userId\n            name\n            nickname\n            gender\n            birth\n            birthStr\n            weight\n            height\n            firstName\n            lastName\n            countryCode\n            city\n            address1\n            address2\n            zipCode\n            emailAddress\n            countryPhoneCode\n            phoneNumber\n            mobilePhoneNumber\n            faxNumber\n            emailConfirm\n            description\n            status\n            lastLoginDate\n            changePasswordDate\n            loginFailCount\n            extra\n            xcoin\n            currentStep\n            totalStep\n            create\n            update\n          }\n          extra\n          listOrder\n          file {\n            __typename\n            id\n            name\n            origName\n            contentType\n            description\n            ipaddress\n            extra\n            create\n            update\n          }\n          create\n          update\n        }\n      }\n      name\n      phoneNumber\n      contacter {\n        __typename\n        id\n        userId\n        name\n        nickname\n        gender\n        birth\n        birthStr\n        weight\n        height\n        firstName\n        lastName\n        countryCode\n        city\n        address1\n        address2\n        zipCode\n        emailAddress\n        countryPhoneCode\n        phoneNumber\n        mobilePhoneNumber\n        faxNumber\n        emailConfirm\n        description\n        status\n        file {\n          __typename\n          id\n          name\n          origName\n          contentType\n          orig {\n            __typename\n            path\n            urlPath\n            urlPathS3\n            size\n            width\n            height\n          }\n          thumbnail {\n            __typename\n            path\n            urlPath\n            urlPathS3\n            size\n            width\n            height\n          }\n          description\n          ipaddress\n          extra\n          create\n          update\n        }\n        lastLoginDate\n        changePasswordDate\n        loginFailCount\n        extra\n        xcoin\n        currentStep\n        totalStep\n        create\n        update\n        roles {\n          __typename\n          id\n          name\n          type\n          defaultRole\n          description\n          status\n          menus {\n            __typename\n            id\n            menuId\n            name\n            status\n            url\n            system\n            listOrder\n            create\n            update\n          }\n          create\n          update\n        }\n        children {\n          __typename\n          id\n          guardian {\n            __typename\n            id\n            userId\n            name\n            nickname\n            gender\n            birth\n            birthStr\n            weight\n            height\n            firstName\n            lastName\n            countryCode\n            city\n            address1\n            address2\n            zipCode\n            emailAddress\n            countryPhoneCode\n            phoneNumber\n            mobilePhoneNumber\n            faxNumber\n            emailConfirm\n            description\n            status\n            lastLoginDate\n            changePasswordDate\n            loginFailCount\n            extra\n            xcoin\n            currentStep\n            totalStep\n            create\n            update\n          }\n          ward {\n            __typename\n            id\n            userId\n            name\n            nickname\n            gender\n            birth\n            birthStr\n            weight\n            height\n            firstName\n            lastName\n            countryCode\n            city\n            address1\n            address2\n            zipCode\n            emailAddress\n            countryPhoneCode\n            phoneNumber\n            mobilePhoneNumber\n            faxNumber\n            emailConfirm\n            description\n            status\n            lastLoginDate\n            changePasswordDate\n            loginFailCount\n            extra\n            xcoin\n            currentStep\n            totalStep\n            create\n            update\n          }\n          flag\n          approval\n          listOrder\n          create\n        }\n        contacts {\n          __typename\n          id\n          me {\n            __typename\n            id\n            userId\n            name\n            nickname\n            gender\n            birth\n            birthStr\n            weight\n            height\n            firstName\n            lastName\n            countryCode\n            city\n            address1\n            address2\n            zipCode\n            emailAddress\n            countryPhoneCode\n            phoneNumber\n            mobilePhoneNumber\n            faxNumber\n            emailConfirm\n            description\n            status\n            lastLoginDate\n            changePasswordDate\n            loginFailCount\n            extra\n            xcoin\n            currentStep\n            totalStep\n            create\n            update\n          }\n          name\n          phoneNumber\n          extra\n          listOrder\n          file {\n            __typename\n            id\n            name\n            origName\n            contentType\n            description\n            ipaddress\n            extra\n            create\n            update\n          }\n          create\n          update\n        }\n      }\n      extra\n      listOrder\n      file {\n        __typename\n        id\n        name\n        origName\n        contentType\n        orig {\n          __typename\n          path\n          urlPath\n          urlPathS3\n          size\n          width\n          height\n        }\n        thumbnail {\n          __typename\n          path\n          urlPath\n          urlPathS3\n          size\n          width\n          height\n        }\n        description\n        ipaddress\n        extra\n        create\n        update\n      }\n      create\n      update\n    }\n  }\n}",
    "ssoMicrosoftM": "mutation ssoMicrosoft($accessToken: String!, $countryCode: String, $phoneNumber: String, $verificationToken: String, $verifyCode: String) {\n  ssoMicrosoft(accessToken: $accessToken, countryCode: $countryCode, phoneNumber: $phoneNumber, verificationToken: $verificationToken, verifyCode: $verifyCode) {\n    __typename\n    id\n    token\n    issueDate\n    expireDate\n    user {\n      __typename\n      id\n      userId\n      name\n      nickname\n      gender\n      birth\n      birthStr\n      weight\n      height\n      firstName\n      lastName\n      countryCode\n      city\n      address1\n      address2\n      zipCode\n      emailAddress\n      countryPhoneCode\n      phoneNumber\n      mobilePhoneNumber\n      faxNumber\n      emailConfirm\n      description\n      status\n      file {\n        __typename\n        id\n        name\n        origName\n        contentType\n        orig {\n          __typename\n          path\n          urlPath\n          urlPathS3\n          size\n          width\n          height\n        }\n        thumbnail {\n          __typename\n          path\n          urlPath\n          urlPathS3\n          size\n          width\n          height\n        }\n        description\n        ipaddress\n        extra\n        create\n        update\n      }\n      lastLoginDate\n      changePasswordDate\n      loginFailCount\n      extra\n      xcoin\n      currentStep\n      totalStep\n      create\n      update\n      roles {\n        __typename\n        id\n        name\n        type\n        defaultRole\n        description\n        status\n        menus {\n          __typename\n          id\n          menuId\n          name\n          status\n          url\n          system\n          listOrder\n          create\n          update\n        }\n        create\n        update\n      }\n      children {\n        __typename\n        id\n        guardian {\n          __typename\n          id\n          userId\n          name\n          nickname\n          gender\n          birth\n          birthStr\n          weight\n          height\n          firstName\n          lastName\n          countryCode\n          city\n          address1\n          address2\n          zipCode\n          emailAddress\n          countryPhoneCode\n          phoneNumber\n          mobilePhoneNumber\n          faxNumber\n          emailConfirm\n          description\n          status\n          file {\n            __typename\n            id\n            name\n            origName\n            contentType\n            description\n            ipaddress\n            extra\n            create\n            update\n          }\n          lastLoginDate\n          changePasswordDate\n          loginFailCount\n          extra\n          xcoin\n          currentStep\n          totalStep\n          create\n          update\n          roles {\n            __typename\n            id\n            name\n            type\n            defaultRole\n            description\n            status\n            create\n            update\n          }\n          children {\n            __typename\n            id\n            flag\n            approval\n            listOrder\n            create\n          }\n          contacts {\n            __typename\n            id\n            name\n            phoneNumber\n            extra\n            listOrder\n            create\n            update\n          }\n        }\n        ward {\n          __typename\n          id\n          userId\n          name\n          nickname\n          gender\n          birth\n          birthStr\n          weight\n          height\n          firstName\n          lastName\n          countryCode\n          city\n          address1\n          address2\n          zipCode\n          emailAddress\n          countryPhoneCode\n          phoneNumber\n          mobilePhoneNumber\n          faxNumber\n          emailConfirm\n          description\n          status\n          file {\n            __typename\n            id\n            name\n            origName\n            contentType\n            description\n            ipaddress\n            extra\n            create\n            update\n          }\n          lastLoginDate\n          changePasswordDate\n          loginFailCount\n          extra\n          xcoin\n          currentStep\n          totalStep\n          create\n          update\n          roles {\n            __typename\n            id\n            name\n            type\n            defaultRole\n            description\n            status\n            create\n            update\n          }\n          children {\n            __typename\n            id\n            flag\n            approval\n            listOrder\n            create\n          }\n          contacts {\n            __typename\n            id\n            name\n            phoneNumber\n            extra\n            listOrder\n            create\n            update\n          }\n        }\n        flag\n        approval\n        listOrder\n        create\n      }\n      contacts {\n        __typename\n        id\n        me {\n          __typename\n          id\n          userId\n          name\n          nickname\n          gender\n          birth\n          birthStr\n          weight\n          height\n          firstName\n          lastName\n          countryCode\n          city\n          address1\n          address2\n          zipCode\n          emailAddress\n          countryPhoneCode\n          phoneNumber\n          mobilePhoneNumber\n          faxNumber\n          emailConfirm\n          description\n          status\n          file {\n            __typename\n            id\n            name\n            origName\n            contentType\n            description\n            ipaddress\n            extra\n            create\n            update\n          }\n          lastLoginDate\n          changePasswordDate\n          loginFailCount\n          extra\n          xcoin\n          currentStep\n          totalStep\n          create\n          update\n          roles {\n            __typename\n            id\n            name\n            type\n            defaultRole\n            description\n            status\n            create\n            update\n          }\n          children {\n            __typename\n            id\n            flag\n            approval\n            listOrder\n            create\n          }\n          contacts {\n            __typename\n            id\n            name\n            phoneNumber\n            extra\n            listOrder\n            create\n            update\n          }\n        }\n        name\n        phoneNumber\n        contacter {\n          __typename\n          id\n          userId\n          name\n          nickname\n          gender\n          birth\n          birthStr\n          weight\n          height\n          firstName\n          lastName\n          countryCode\n          city\n          address1\n          address2\n          zipCode\n          emailAddress\n          countryPhoneCode\n          phoneNumber\n          mobilePhoneNumber\n          faxNumber\n          emailConfirm\n          description\n          status\n          file {\n            __typename\n            id\n            name\n            origName\n            contentType\n            description\n            ipaddress\n            extra\n            create\n            update\n          }\n          lastLoginDate\n          changePasswordDate\n          loginFailCount\n          extra\n          xcoin\n          currentStep\n          totalStep\n          create\n          update\n          roles {\n            __typename\n            id\n            name\n            type\n            defaultRole\n            description\n            status\n            create\n            update\n          }\n          children {\n            __typename\n            id\n            flag\n            approval\n            listOrder\n            create\n          }\n          contacts {\n            __typename\n            id\n            name\n            phoneNumber\n            extra\n            listOrder\n            create\n            update\n          }\n        }\n        extra\n        listOrder\n        file {\n          __typename\n          id\n          name\n          origName\n          contentType\n          description\n          ipaddress\n          extra\n          create\n          update\n        }\n        create\n        update\n      }\n    }\n    app {\n      __typename\n      id\n      name\n      packageName\n      apiKey\n      apiSecret\n      terminalType\n      description\n      status\n      file {\n        __typename\n        id\n        name\n        origName\n        contentType\n        description\n        ipaddress\n        extra\n        create\n        update\n      }\n      versions {\n        __typename\n        id\n        version\n        requireUpdate\n        downloadUrl\n        description\n        create\n        update\n      }\n      create\n      update\n    }\n    w360 {\n      __typename\n      token\n      secret\n      qid\n    }\n  }\n}",
}
STEP_M: Dict[str, str] = {
    "addM": "mutation AddStep($stepCount: Int) {\n  addStep(stepCount: $stepCount)\n}",
}
UTILS_M: Dict[str, str] = {
    "issueCodeM": "mutation IssueCode($purpose: VerificationType!, $countryCode: String!, $phoneNumber: String!, $emailAddress: String) {\n  issueCode(purpose: $purpose, countryCode: $countryCode, phoneNumber: $phoneNumber, emailAddress: $emailAddress) {\n    __typename\n    registered\n    sent\n    ttl\n  }\n}",
    "resetPasswordM": "mutation ResetPassword($countryPhoneNumber : String!, $phoneNumber : String!, $verifyCode : String!, $password : String!) {\n  resetPassword(countryPhoneNumber: $countryPhoneNumber, phoneNumber: $phoneNumber, verifyCode: $verifyCode, password: $password) {\n    __typename\n    alreadyRegistered\n    resetSuccess\n  }\n}",
    "issueCodeWithEmailM": "mutation IssueCodeWithEmail($purpose: VerificationType!, $emailAddress: String!) {\n  issueCodeWithEmail(purpose: $purpose, emailAddress: $emailAddress) {\n    __typename\n    registered\n    sent\n    ttl\n  }\n}",
    "issueEmailAndPhoneCodeM": "mutation issueEmailAndPhoneCode($purpose: EmailAndPhoneVerificationType!, $countryCode: String, $phoneNumber: String, $email: String, $previousToken: String) {\n  issueEmailAndPhoneCode(purpose: $purpose, countryCode: $countryCode, phoneNumber: $phoneNumber, email: $email, previousToken: $previousToken) {\n    __typename\n    token\n  }\n}",
    "issueEmailCodeM": "mutation IssueEmailCode($purpose: EmailAndPhoneVerificationType!, $email: String, $previousToken: String) {\n  issueEmailCode(purpose: $purpose, email: $email, previousToken: $previousToken) {\n    __typename\n    token\n  }\n}",
    "resetPasswordWithEmailM": "mutation ResetPasswordWithEmail($emailAddress: String!, $verifyCode: String!, $password: String!) {\n  resetPasswordWithEmail(emailAddress: $emailAddress, verifyCode: $verifyCode, password: $password) {\n    __typename\n    alreadyRegistered\n    resetSuccess\n  }\n}",
    "resetPasswordWithEmailOrPhoneM": "mutation resetPasswordWithEmailOrPhone($countryPhoneNumber: String, $phoneNumber: String, $verifyCode: String!, $password: String!, $verificationToken: String!, $emailAddress: String) {\n  resetPasswordWithEmailOrPhone(countryPhoneNumber: $countryPhoneNumber, phoneNumber: $phoneNumber, verifyCode: $verifyCode, password: $password, verificationToken: $verificationToken, emailAddress: $emailAddress) {\n    __typename\n    alreadyRegistered\n    resetSuccess\n  }\n}",
    "userUnitSettingsM": "mutation UserUnitSettings($uid: String!, $unit_setting: Int!) {\n  userUnitSettings(uid: $uid, unit_setting: $unit_setting)\n}",
}
WATCH_M: Dict[str, str] = {
    "confirmM": "mutation ConfirmWatch($deviceId : String!, $qid : String!) {\n  confirmWatch(deviceId: $deviceId, qid: $qid)\n}",
    "createM": "mutation CreateWatch($countryCode : String!, $phoneNumber: String!, $qrCode : String!, $qrt : String!, $qrc : String!) {\n  createWatch(countryCode: $countryCode, phoneNumber: $phoneNumber, qrCode: $qrCode, qrt: $qrt, qrc: $qrc) {\n    __typename\n    ...WatchFragment\n  }\n}\nfragment WatchFragment on Watch {\n  __typename\n  id\n  group {\n    __typename\n    ...WatchGroupFragment\n  }\n  vendor {\n    __typename\n    ...VendorFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  name\n  swKey\n  groupName\n  os\n  osName\n  osVersion\n  brand\n  phoneNumber\n  qrCode\n  countryPhoneCode\n  onlineStatus\n  status\n  activated\n  extra\n  create\n  update\n}\nfragment WatchGroupFragment on WatchGroup {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment VendorFragment on Vendor {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}",
    "deleteAlertM": "mutation deleteAlert($rid: String!) {\n  deleteAlert(rid: $rid)\n}",
    "denyM": "mutation DenyWatch($deviceId: String!, $requestId: String!) {\n  denyWatch(deviceId: $deviceId, requestId: $requestId)\n}",
    "modifyAlertM": "mutation modifyAlert($uid: String!, $remind: YesOrNo!) {\n  modifyAlert(uid: $uid, remind: $remind)\n}",
    "rebootM": "mutation reboot($uid : String!) {\n  reboot(uid: $uid)\n}",
    "removeM": "mutation RemoveWatch($uid : String!) {\n  removeWatch(uid: $uid)\n}",
    "resetM": "mutation ResetWatch($uid: String!) {\n  resetWatch(uid: $uid)\n}",
    "setInfoM": "mutation SetWatchInfo($uid: String, $name : String, $qr: String, $phoneNumber : String, $countryCode : String) {\n  setWatchInfo(uid: $uid, name: $name, qr: $qr, phoneNumber: $phoneNumber, countryCode: $countryCode)\n}",
    "shutdownM": "mutation ShutDown($uid: String!) {\n  shutDown(uid: $uid)\n}",
    "updateM": "mutation UpdateWatch($uid : String!, $countryCode : String, $phoneNumber : String) {\n  updateWatch(uid: $uid, countryCode: $countryCode, phoneNumber: $phoneNumber)\n}",
    "addAlarmM": "mutation AddAlarm($uid: String!, $name: String, $occurMin: Int!, $start: Int!, $end: Int, $weekRepeat: String!, $description: String, $extra: JSON, $timeZone: String) {\n  addAlarm(uid: $uid, name: $name, occurMin: $occurMin, start: $start, end: $end, weekRepeat: $weekRepeat, description: $description, extra: $extra, timeZone: $timeZone) {\n    __typename\n    ...WatchAlarmFragment\n  }\n}\nfragment WatchAlarmFragment on WatchAlarm {\n  __typename\n  id\n  vendorName\n  vendorId\n  watch {\n    __typename\n    ...WatchFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  name\n  start\n  end\n  weekRepeat\n  occurMin\n  description\n  extra\n  start\n  create\n  update\n  status\n}\nfragment WatchFragment on Watch {\n  __typename\n  id\n  group {\n    __typename\n    ...WatchGroupFragment\n  }\n  vendor {\n    __typename\n    ...VendorFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  name\n  swKey\n  groupName\n  os\n  osName\n  osVersion\n  brand\n  phoneNumber\n  qrCode\n  countryPhoneCode\n  onlineStatus\n  status\n  activated\n  extra\n  create\n  update\n}\nfragment WatchGroupFragment on WatchGroup {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment VendorFragment on Vendor {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}",
    "modifyAlarmM": "mutation ModifyAlarm($alarmId: String!, $name: String, $occurMin: Int, $start: Int, $end: Int, $weekRepeat: String, $description: String, $status: NormalStatus, $extra: JSON, $timeZone: String) {\n  modifyAlarm(alarmId: $alarmId, name: $name, occurMin: $occurMin, status: $status, end: $end, weekRepeat:     $weekRepeat, description: $description, start: $start, extra: $extra, timeZone: $timeZone)\n}",
    "removeAlarmM": "mutation RemoveAlarm($alarmId : String!) {\n  removeAlarm(alarmId: $alarmId)\n}",
    "deleteChatMessageM": "mutation DeleteChatMessage($uid: String!, $msgId: String!) {\n  deleteMsg(uid: $uid, msgId: $msgId)\n}",
    "sendChatEmoticonM": "mutation SendChatEmoticon($uid : String!, $emoticonId : ChatEmoticonType!) {\n  sendChatEmoticon(uid: $uid, emoticonId: $emoticonId)\n}",
    "sendChatImageM": "mutation SendChatImage($uid: String!, $format: String!, $hash: String!, $hashBody: String!, $tm:String) {\n  sendChatImage(uid: $uid, format: $format, hash: $hash, hashBody: $hashBody, tm: $tm)\n}",
    "sendChatMp3M": "mutation SendChatMp3($uid: String!, $fileName: String!, $duration: Int, $hash: String, $hashBody: String!, $tm: String) {\n  sendChatMp3(uid: $uid, fileName: $fileName, duration: $duration, hash: $hash, hashBody: $hashBody, tm: $tm)\n}",
    "sendChatTextM": "mutation SendChatText($uid : String!, $text : String!) {\n  sendChatText(uid: $uid, text: $text)\n}",
    "sendChatVoiceM": "mutation SendChatVoice($uid : String!, $duration : Int!, $hash:String!, $hashBody:String!, $tm:String) {\n  sendChatVoice(uid: $uid, duration: $duration, hash: $hash, hashBody: $hashBody, tm: $tm)\n}",
    "setReadChatMsgM": "mutation setReadChatMsg($uid: String!, $msgId : String, $id : String) {\n  setReadChatMsg(uid: $uid, msgId: $msgId, id: $id)\n}",
    "addContactM": "mutation AddContact($uid : String!, $contactName : String, $countryPhoneNumber : String, $phoneNumber : String, $fileId : String) {\n  addContact(uid: $uid, contactName: $contactName, countryPhoneNumber: $countryPhoneNumber, phoneNumber: $phoneNumber, fileId: $fileId) {\n    __typename\n    ...ContactFragment\n  }\n}\nfragment ContactFragment on XPContact {\n  __typename\n  id\n  ownUser {\n    __typename\n    ...SimpleUserFragment\n  }\n  name\n  countryPhoneNumber\n  phoneNumber\n  contactUser {\n    __typename\n    ...SimpleUserFragment\n  }\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  approval\n  guardianType\n  create\n  update\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}",
    "modifyContactM": "mutation ModifyContact($contactId : String!, $contactName : String, $fileId : String, $isAdmin : Boolean) {\n  modifyContact(contactId: $contactId, contactName: $contactName, fileId: $fileId, isAdmin: $isAdmin)\n}",
    "removeContactM": "mutation RemoveContact($contactId : String!) {\n  removeContact(contactId: $contactId)\n}",
    "addSilentTimeM": "mutation AddSilentTime($uid: String!, $start: Int!, $end: Int!, $weekRepeat: String!, $description: String, $extra: JSON) {\n  addSilentTime(uid: $uid, start: $start, end: $end, weekRepeat: $weekRepeat, description: $description, extra: $extra) {\n    __typename\n    ...SlientModeFragment\n  }\n}\nfragment SlientModeFragment on SilentMode {\n  __typename\n  id\n  vendorName\n  vendorId\n  watch {\n    __typename\n    ...WatchFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  start\n  end\n  weekRepeat\n  description\n  extra\n  start\n  create\n  update\n}\nfragment WatchFragment on Watch {\n  __typename\n  id\n  group {\n    __typename\n    ...WatchGroupFragment\n  }\n  vendor {\n    __typename\n    ...VendorFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  name\n  swKey\n  groupName\n  os\n  osName\n  osVersion\n  brand\n  phoneNumber\n  qrCode\n  countryPhoneCode\n  onlineStatus\n  status\n  activated\n  extra\n  create\n  update\n}\nfragment WatchGroupFragment on WatchGroup {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment VendorFragment on Vendor {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}",
    "modifySilentTimeM": "mutation ModifySilentTime($silentId: String!, $start: Int, $end: Int, $weekRepeat: String, $description: String, $extra: JSON) {\n  modifySilentTime(silentId: $silentId, start: $start, end: $end, weekRepeat: $weekRepeat, description: $description, extra: $extra)\n}",
    "removeSilentTimeM": "mutation RemoveSilentTime($silentId : String!) {\n  removeSilentTime(silentId: $silentId)\n}",
    "setEnableSlientTimeM": "mutation SetEnableSlientTime($silentId : String!, $status: NormalStatus!) {\n  setEnableSilentTime(silentId: $silentId, status: $status)\n}",
    "addSafeZoneM": "mutation AddSafeZone($uid : String!, $groupId: String!, $name : String!, $lat: String!, $lng: String!, $rad: Int!, $address : String!) {\n  addSafeZone(uid: $uid, groupId: $groupId, name: $name, lat: $lat, lng: $lng, rad: $rad, address: $address) {\n    __typename\n    ...SafeZoneFragment\n  }\n}\nfragment SafeZoneFragment on SafeZone {\n  __typename\n  id\n  vendorName\n  vendorId\n  groupName\n  watch {\n    __typename\n    ...WatchFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  group {\n    __typename\n    ...SafeZoneGroupFragment\n  }\n  name\n  lat\n  lng\n  rad\n  address\n  description\n  extra\n  create\n  update\n}\nfragment WatchFragment on Watch {\n  __typename\n  id\n  group {\n    __typename\n    ...WatchGroupFragment\n  }\n  vendor {\n    __typename\n    ...VendorFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  name\n  swKey\n  groupName\n  os\n  osName\n  osVersion\n  brand\n  phoneNumber\n  qrCode\n  countryPhoneCode\n  onlineStatus\n  status\n  activated\n  extra\n  create\n  update\n}\nfragment WatchGroupFragment on WatchGroup {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment VendorFragment on Vendor {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}\nfragment SafeZoneGroupFragment on SafeZoneGroup {\n  __typename\n  id\n  name\n  status\n  description\n  create\n  update\n}",
    "removeSafeZoneM": "mutation RemoveSafeZone($zoneId : String!) {\n  removeSafeZone(zoneId: $zoneId)\n}",
    "addWiFiM": "mutation AddWiFi($uid : String!, $ssid : String!, $name : String, $password : String) {\n  addWiFi(uid: $uid, ssid: $ssid, name: $name, password: $password) {\n    __typename\n    ...WiFiFragment\n  }\n}\nfragment WiFiFragment on WiFi {\n  __typename\n  id\n  name\n  ssid\n  password\n  state\n  create\n  update\n  user {\n    __typename\n    ...UserFragment\n  }\n  watch {\n    __typename\n    ...WatchFragment\n  }\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}\nfragment WatchFragment on Watch {\n  __typename\n  id\n  group {\n    __typename\n    ...WatchGroupFragment\n  }\n  vendor {\n    __typename\n    ...VendorFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  name\n  swKey\n  groupName\n  os\n  osName\n  osVersion\n  brand\n  phoneNumber\n  qrCode\n  countryPhoneCode\n  onlineStatus\n  status\n  activated\n  extra\n  create\n  update\n}\nfragment WatchGroupFragment on WatchGroup {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment VendorFragment on Vendor {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}",
    "modifyWiFiM": "mutation ModifyWiFi($wifiId : String!, $name : String, $password : String) {\n  modifyWiFi(wifiId: $wifiId, name: $name, password: $password)\n}",
    "removeWiFiM": "mutation RemoveWiFi($wifiId : String!) {\n  removeWiFi(wifiId: $wifiId)\n}",
    "submitIncorrectLocationDataM": "mutation SubmitIncorrectLocationData($uid: String!, $lat: String!, $lng: String!, $timestamp: String!) {\n  submitIncorrectLocationData(uid: $uid, lat: $lat, lng: $lng, timestamp : $timestamp)\n}",
}
WATCHGROUP_M: Dict[str, str] = {
    "addM": "mutation AddWatchGroup($name: String!, $extra: JSON, $description: String) {\n  addWatchGroup(name: $name, extra: $extra, description: $description) {\n    __typename\n    ...WatchGroupFragment\n  }\n}\nfragment WatchGroupFragment on WatchGroup {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}",
    "modifyM": "mutation ModifyWatchGroup($id: String, $name: String!, $extra: JSON, $description: String, $status: NormalStatus) {\n  modifyWatchGroup(id: $id, name: $name, extra: $extra, description: $description, status: $status)\n}",
    "removeM": "mutation RemoveWatchGroup($id: String) {\n  removeWatchGroup(id: $id)\n}",
}
XCOIN_M: Dict[str, str] = {
    "transferM": "mutation TransferCoin($src: String!, $dest: String!, $coin: Int!) {\n  transferCoin(src: $src, dest: $dest, coin: $coin)\n}",
}
XMOVE_M: Dict[str, str] = {
    "addAlarmReminderM": "mutation AddAlarmReminder($uid: String!, $occurMin: Int!, $start: Int!, $end: Int, $weekRepeat: String!, $threshold: Int!, $hour: Int!, $minute: Int!, $interval: Int!, $extra: JSON) {\n  addAlarmReminder(uid: $uid, occurMin: $occurMin, start: $start, end: $end, weekRepeat: $weekRepeat, threshold: $threshold, hour: $hour, minute: $minute, interval: $interval, extra: $extra) {\n    __typename\n    ...WatchReminderFragment\n  }\n}\nfragment WatchReminderFragment on WatchReminder {\n  __typename\n  id\n  vendorName\n  vendorId\n  watch {\n    __typename\n    ...WatchFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  start\n  end\n  weekRepeat\n  occurMin\n  extra\n  start\n  create\n  update\n  status\n}\nfragment WatchFragment on Watch {\n  __typename\n  id\n  group {\n    __typename\n    ...WatchGroupFragment\n  }\n  vendor {\n    __typename\n    ...VendorFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  name\n  swKey\n  groupName\n  os\n  osName\n  osVersion\n  brand\n  phoneNumber\n  qrCode\n  countryPhoneCode\n  onlineStatus\n  status\n  activated\n  extra\n  create\n  update\n}\nfragment WatchGroupFragment on WatchGroup {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment VendorFragment on Vendor {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}",
    "createBTWatchM": "mutation CreateBTWatch($qrCode : String!, $qrt : String!, $qrc : String!) {\n  createBTWatch(qrCode: $qrCode, qrt: $qrt, qrc: $qrc) {\n    __typename\n    ...WatchBTFragment\n  }\n}\nfragment WatchBTFragment on Watch {\n  __typename\n  id\n  group {\n    __typename\n    ...WatchBTGroupFragment\n  }\n  vendor {\n    __typename\n    ...VendorBTFragment\n  }\n  user {\n    __typename\n    ...UserFragment\n  }\n  name\n  os\n  osName\n  osVersion\n  brand\n  phoneNumber\n  qrCode\n  countryPhoneCode\n  onlineStatus\n  status\n  extra\n  create\n  update\n}\nfragment WatchBTGroupFragment on WatchGroup {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment VendorBTFragment on Vendor {\n  __typename\n  id\n  name\n  status\n  extra\n  description\n  create\n  update\n}\nfragment UserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  birth\n  birthStr\n  weight\n  height\n  countryCode\n  emailAddress\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  emailConfirm\n  status\n  file {\n    __typename\n    ...FileFragment\n  }\n  extra\n  xcoin\n  currentStep\n  totalStep\n  create\n  update\n  children {\n    __typename\n    id\n    guardian {\n      __typename\n      ...SimpleUserFragment\n    }\n    ward {\n      __typename\n      ...SimpleUserFragment\n    }\n  }\n}\nfragment FileFragment on File {\n  __typename\n  id\n  name\n}\nfragment SimpleUserFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  gender\n  countryCode\n  countryPhoneCode\n  phoneNumber\n  mobilePhoneNumber\n  file {\n    __typename\n    ...FileFragment\n  }\n  xcoin\n  currentStep\n  totalStep\n  contacts {\n    __typename\n    ...ContactsFragment\n  }\n}\nfragment ContactsFragment on Contact {\n  __typename\n  id\n  me {\n    __typename\n    ...ContactorFragment\n  }\n  contacter {\n    __typename\n    ...ContactorFragment\n  }\n  phoneNumber\n  extra\n  listOrder\n  file {\n    __typename\n    ...FileFragment\n  }\n  create\n  update\n}\nfragment ContactorFragment on User {\n  __typename\n  id\n  userId\n  name\n  nickname\n  countryCode\n  countryPhoneCode\n  mobilePhoneNumber\n  phoneNumber\n}",
    "removeAlarmReminderM": "mutation removeAlarmReminder($reminderId : String!) {\n  removeAlarmReminder(reminderId: $reminderId)\n}",
    "resetBTWatchM": "mutation ResetBTWatch($uid: String!) {\n  resetBTWatch(uid: $uid)\n}",
    "updateAlarmReminderM": "mutation updateAlarmReminder($reminderId: String!, $occurMin: Int, $start: Int, $end: Int, $weekRepeat: String, $threshold: Int, $hour: Int, $minute: Int, $interval: Int, $status: NormalStatus, $extra: JSON) {\n  updateAlarmReminder(reminderId: reminderId, occurMin: $occurMin, start: $start, end: $end, weekRepeat: $weekRepeat, threshold: $threshold, hour: $hour, minute: $minute, interval: $interval, status: $status, extra: $extra)\n}",
}



from __future__ import annotations


class Error(Exception):
    pass


class NoAdminError(Error):
    def __init__(self) -> None:
        super().__init__()

    def __str__(self) -> str:
        return "no Admin"


class ChildNoError(Error):
    def __init__(self) -> None:
        super().__init__()

    def __str__(self) -> str:
        return "Child phonenumber or Watch ID not found!"


class FunctionError(Error):
    # FunctionError(sys._getframe().f_code.co_name)
    def __init__(self, fnc: str) -> None:
        self.fnc = fnc
        super().__init__(self.fnc)

    def __str__(self) -> str:
        return "Xplora API call finally failed with response: {0}".format(self.fnc)


class LoginError(Error):
    def __init__(self, message: str, res: int = 1) -> None:
        self.message = message
        self.res = res
        super().__init__(self.message, self.res)

    def __str__(self) -> str:
        return f"{self.message} - {self.res}"



from __future__ import annotations

from typing import Any, Dict

from python_graphql_client import GraphqlClient

from .const import ENDPOINT
from .exception_classes import LoginError, NoAdminError
from .handler_gql import HandlerGQL
from .status import NormalStatus, YesOrNo

from . import gql_mutations as gm
from . import gql_queries as gq


class GQLHandler(HandlerGQL):
    def __init__(
        self,
        countryPhoneNumber: str,
        phoneNumber: str,
        password: str,
        userLang: str,
        timeZone: str,
    ) -> None:
        super().__init__(countryPhoneNumber, phoneNumber, password, userLang, timeZone)

    def runGqlQuery(self, query: str, variables: Dict[str, Any]) -> Dict[str, Any]:
        if query is None:
            raise Exception("GraphQL guery string MUST NOT be empty!")
        # Add Xplora® API headers
        requestHeaders = self.getRequestHeaders("application/json; charset=UTF-8")
        # create GQLClient
        gqlClient = GraphqlClient(endpoint=ENDPOINT, headers=requestHeaders)
        # execute QUERY|MUTATION
        data: Dict[str, Any] = gqlClient.execute(query=query, variables=variables)
        return data

    def runAuthorizedGqlQuery(self, query: str, variables: Dict[str, Any]) -> Dict[str, Any]:
        if self.accessToken is None:
            raise Exception("You must first login to the Xplora® API.")
        # Run GraphQL query and return
        return self.runGqlQuery(query, variables)

    def login(self) -> Dict[str, Any]:
        data: Dict[str, Any] = self.runGqlQuery(gm.SIGN_M.get("issueTokenM", ""), self.variables).get("data", {})
        if data["issueToken"] is None:
            # Login failed.
            raise LoginError("Login to Xplora® API failed. Check your input!")
        self.issueToken = data["issueToken"]

        # Login succeeded
        self.sessionId = self.issueToken["id"]
        self.userId = self.issueToken["user"]["id"]
        self.accessToken = self.issueToken["token"]
        self.issueDate = self.issueToken["issueDate"]
        self.expireDate = self.issueToken["expireDate"]

        if self.issueToken["app"] is not None:
            # Update API_KEY and API_SECRET?
            if self.issueToken["app"]["apiKey"]:
                self._API_KEY = self.issueToken["app"]["apiKey"]
            if self.issueToken["app"]["apiSecret"]:
                self._API_SECRET = self.issueToken["app"]["apiSecret"]
        return self.issueToken

    def isAdmin(self, wuid: str, query: str, variables: Dict[str, Any], key: str) -> bool:
        contacts: Dict[str, Any] = self.getWatchUserContacts(wuid)
        for contact in contacts["contacts"]["contacts"]:
            try:
                id = contact["contactUser"]["id"]
            except KeyError and TypeError:
                id = None
            if self.userId == id:
                if contact["guardianType"] == "FIRST":
                    data: Dict[str, Any] = self.runAuthorizedGqlQuery(query, variables).get("data", {})
                    return data.get(key, False)
        raise NoAdminError()

    ########## SECTION QUERY start ##########

    def askWatchLocate(self, wuid: str) -> Dict[str, Any]:
        res: Dict[str, Any] = self.runAuthorizedGqlQuery(gq.WATCH_Q.get("askLocateQ", ""), {"uid": wuid}).get("data", {})
        if res["askWatchLocate"] is not None:
            return res
        return {"askWatchLocate": False}

    def getWatchUserContacts(self, wuid: str) -> Dict[str, Any]:
        # Contacts from ownUser
        return self.runAuthorizedGqlQuery(gq.WATCH_Q.get("contactsQ", ""), {"uid": wuid}).get("data", {})

    def getWatches(self, wuid: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.WATCH_Q.get("watchesQ", ""), {"uid": wuid}).get("data", {})

    def getSWInfo(self, qrCode: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.WATCH_Q.get("checkByQrCodeQ", ""), {"qrCode": qrCode}).get("data", {})

    def getWatchState(self, qrCode: str, qrt: str = "", qrc: str = "") -> Dict[str, Any]:
        vari = {}
        if qrCode != "":
            vari["qrCode"] = qrCode
        if qrt != "":
            vari["qrt"] = qrt
        if qrc != "":
            vari["qrc"] = qrc
        return self.runAuthorizedGqlQuery(gq.WATCH_Q.get("stateQ", ""), vari).get("data", {})

    def getWatchLastLocation(self, wuid: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.WATCH_Q.get("locateQ", ""), {"uid": wuid}).get("data", {})

    def trackWatch(self, wuid: str) -> Dict[str, Any]:
        # tracking time - seconds
        res: Dict[str, Any] = self.runAuthorizedGqlQuery(gq.WATCH_Q.get("trackQ", ""), {"uid": wuid}).get("data", {})
        if res.get("trackWatch", {"trackWatch": -1}):
            return res
        return {"trackWatch": -1}

    def getAlarmTime(self, wuid: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.WATCH_Q.get("alarmsQ", ""), {"uid": wuid}).get("data", {})

    def getWifi(self, wuid: str) -> Dict[str, Any]:
        # without function?
        return self.runAuthorizedGqlQuery(gq.WATCH_Q.get("getWifisQ", ""), {"uid": wuid}).get("data", {})

    def unReadChatMsgCount(self, wuid: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.WATCH_Q.get("unReadChatMsgCountQ", ""), {"uid": wuid}).get("data", {})

    def safeZones(self, wuid: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.WATCH_Q.get("safeZonesQ", ""), {"uid": wuid}).get("data", {})

    def safeZoneGroups(self) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.WATCH_Q.get("safeZoneGroupsQ", ""), {}).get("data", {})

    def silentTimes(self, wuid: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.WATCH_Q.get("silentTimesQ", ""), {"uid": wuid}).get("data", {})

    def chats(self, wuid: str, offset: int = 0, limit: int = 100, msgId: str = "") -> Dict[str, Any]:
        # ownUser id
        return self.runAuthorizedGqlQuery(
            gq.WATCH_Q.get("chatsQ", ""), {"uid": wuid, "offset": offset, "limit": limit, "msgId": msgId}
        ).get("data", {})

    def fetchChatImage(self, wuid: str, msgId: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.WATCH_Q.get("fetchChatImageQ", ""), {"uid": wuid, "msgId": msgId}).get("data", {})

    def fetchChatMp3(self, wuid: str, msgId: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.WATCH_Q.get("fetchChatMp3Q", ""), {"uid": wuid, "msgId": msgId}).get("data", {})

    def fetchChatShortVideo(self, wuid: str, msgId: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.WATCH_Q.get("fetchChatShortVideoQ", ""), {"uid": wuid, "msgId": msgId}).get(
            "data", {}
        )

    def fetchChatVoice(self, wuid: str, msgId: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.WATCH_Q.get("fetchChatVoiceQ", ""), {"uid": wuid, "msgId": msgId}).get("data", {})

    def watchImei(self, imei: str, qrCode: str, deviceKey: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(
            gq.WATCH_Q.get("imeiQ", ""), {"imei": imei, "qrCode": qrCode, "deviceKey": deviceKey}
        ).get("data", {})

    def getWatchLocHistory(self, wuid: str, date: int, tz: str, limit: int) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(
            gq.WATCH_Q.get("locHistoryQ", ""), {"uid": wuid, "date": date, "tz": tz, "limit": limit}
        ).get("data", {})

    def watchesDynamic(self) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.WATCH_Q.get("watchesDynamicQ", ""), {}).get("data", {})

    def coinHistory(self, wuid: str, start: int, end: int, type: str, offset: int, limit: int) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(
            gq.XCOIN_Q.get("historyQ", ""),
            {"uid": wuid, "start": start, "end": end, "type": type, "offset": offset, "limit": limit},
        ).get("data", {})

    def reminders(self, wuid: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.XMOVE_Q.get("remindersQ", ""), {"uid": wuid}).get("data", {})

    def groups(self, isCampaign: bool) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.CARD_Q.get("groupsQ", ""), {"isCampaign": isCampaign}).get("data", {})

    def dynamic(self) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.CARD_Q.get("dynamicQ", ""), {}).get("data", {})

    def staticCard(self) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.CARD_Q.get("staticQ", ""), {}).get("data", {})

    def familyInfo(self, wuid: str, watchId: str, tz: str, date: int) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(
            gq.FAMILY_Q.get("infoQ", ""), {"uid": wuid, "watchId": watchId, "tz": tz, "date": date}
        ).get("data", {})

    def getMyTotalInfo(
        self, wuid: str, tz: str, date: int, start: int, end: int, type: str, offset: int, limit: int
    ) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(
            gq.MYINFO_Q.get("getMyTotalInfoQ", ""),
            {"uid": wuid, "tz": tz, "date": date, "start": start, "end": end, "type": type, "offset": offset, "limit": limit},
        ).get("data", {})

    def myInfoWithCoinHistory(
        self, wuid: str, start: int, end: int, tz: str, type: str, offset: int, limit: int
    ) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(
            gq.MYINFO_Q.get("coinHistoryQ", ""),
            {"uid": wuid, "start": start, "end": end, "tz": tz, "type": type, "offset": offset, "limit": limit},
        ).get("data", {})

    def getMyInfo(self) -> Dict[str, Any]:
        # Profil from login Account
        return self.runAuthorizedGqlQuery(gq.MYINFO_Q.get("readQ", ""), {}).get("data", {})

    def readCampaignProfile(self) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(
            gq.MYINFO_Q.get("readCampaignProfileQ", ""),
            {},
        ).get("data", {})

    def getReviewStatus(self, wuid: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.REVIEW_Q.get("getStatusQ", ""), {"uid": wuid}).get("data", {})

    def getWatchUserSteps(self, wuid: str, tz: str, date: int) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.STEP_Q.get("userQ", ""), {"uid": wuid, "tz": tz, "date": date}).get("data", {})

    def countries(self) -> Dict[str, Any]:
        # Country Support
        return self.runAuthorizedGqlQuery(gq.UTILS_Q.get("countriesQ", ""), {}).get("data", {})

    def subscribedCampaign(self, wuid: str, needDetail: bool = False) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.CAMPAIGN_Q.get("subscribedQ", ""), {"uid": wuid, "needDetail": needDetail}).get(
            "data", {}
        )

    def avatars(self, id: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.CAMPAIGN_Q.get("avatarsQ", ""), {"id": id}).get("data", {})

    def getFollowRequestWatchCount(self) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.CAMPAIGN_Q.get("followRequestWatchCountQ", ""), {}).get("data", {})

    def campaigns(self, id: str, categoryId: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.CAMPAIGN_Q.get("campaignsQ", ""), {"id": id, "categoryId": categoryId}).get(
            "data", {}
        )

    def isSubscribed(self, id: str, wuid: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.CAMPAIGN_Q.get("isSubscribedQ", ""), {"id": id, "uid": wuid}).get("data", {})

    def subscribed(self, wuid: str, needDetail: bool) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.CAMPAIGN_Q.get("subscribedQ", ""), {"uid": wuid, "needDetail": needDetail}).get(
            "data", {}
        )

    def ranks(self, campaignId: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.CAMPAIGN_Q.get("ranksQ", ""), {"campaignId": campaignId}).get("data", {})

    def campaignUserProfiles(self) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.CAMPAIGN_Q.get("campaignUserProfilesQ", ""), {}).get("data", {})

    def conv360IDToO2OID(self, qid: str, deviceId: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.QUERY.get("conv360IDToO2OIDQ", ""), {"qid": qid, "deviceId": deviceId}).get(
            "data", {}
        )

    def watchGroups(self, id: str = "") -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gq.WATCHGROUP_Q.get("watchGroupsQ", ""), {"id": id}).get("data", {})

    ########## SECTION QUERY end ##########

    ########## SECTION MUTATION start ##########

    def sendText(self, wuid: str, text: str) -> bool:
        # ownUser id
        if (
            self.runAuthorizedGqlQuery(gm.WATCH_M.get("sendChatTextM", ""), {"uid": wuid, "text": text}).get("data", {})[
                "sendChatText"
            ]
            is not None
        ):
            return True
        return False

    def addStep(self, stepCount: int) -> Dict[str, bool]:
        return self.runAuthorizedGqlQuery(gm.STEP_M.get("addM", ""), {"stepCount": stepCount}).get("data", {})

    def shutdown(self, wuid: str) -> bool:
        # ownUser id
        return self.isAdmin(wuid, gm.WATCH_M.get("shutdownM", ""), {"uid": wuid}, "shutDown")

    def reboot(self, wuid: str) -> bool:
        # ownUser id
        return self.isAdmin(wuid, gm.WATCH_M.get("rebootM", ""), {"uid": wuid}, "reboot")

    def modifyAlert(self, id: str, yesOrNo: YesOrNo) -> Dict[str, Any]:
        # function?
        return self.runAuthorizedGqlQuery(gm.WATCH_M.get("modifyAlertM", ""), {"uid": id, "remind": yesOrNo})

    def setEnableSlientTime(self, silentId: str, status: str = NormalStatus.ENABLE.value) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(
            gm.WATCH_M.get("setEnableSlientTimeM", ""), {"silentId": silentId, "status": status}
        ).get("data", {})

    def setEnableAlarmTime(self, alarmId: str, status: str = NormalStatus.ENABLE.value) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gm.WATCH_M.get("modifyAlarmM", ""), {"alarmId": alarmId, "status": status}).get(
            "data", {}
        )

    def setReadChatMsg(self, wuid: str, msgId: str, id: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(gm.WATCH_M.get("setReadChatMsgM", ""), {"uid": wuid, "msgId": msgId, "id": id}).get(
            "data", {}
        )

    def submitIncorrectLocationData(self, wuid: str, lat: str, lng: str, timestamp: str) -> Dict[str, Any]:
        return self.runAuthorizedGqlQuery(
            gm.WATCH_M.get("submitIncorrectLocationDataM", ""), {"uid": wuid, "lat": lat, "lng": lng, "timestamp": timestamp}
        ).get("data", {})

    ########## SECTION MUTATION end ##########



from __future__ import annotations

import logging

from datetime import datetime
from time import time
from typing import Any, Dict, List

from .const import VERSION, VERSION_APP
from .exception_classes import LoginError, NoAdminError
from .gql_handler import GQLHandler
from .pyxplora import PyXplora
from .status import LocationType, NormalStatus, WatchOnlineStatus

_LOGGER = logging.getLogger(__name__)

_LIST_DICT: List[Dict[str, Any]] = []


class PyXploraApi(PyXplora):
    def __init__(
        self,
        countrycode: str,
        phoneNumber: str,
        password: str,
        userLang: str,
        timeZone: str,
        childPhoneNumber: List[str] = [],
    ) -> None:
        super().__init__(countrycode, phoneNumber, password, userLang, timeZone, childPhoneNumber)

    def __login(self, forceLogin: bool = False) -> Dict[Any, Any]:
        if not self._isConnected() or self._hasTokenExpired() or forceLogin:
            try:
                self._logoff()
                self._gqlHandler: GQLHandler = GQLHandler(
                    self._countrycode,
                    self._phoneNumber,
                    self._password,
                    self._userLang,
                    self._timeZone,
                )
                if self._gqlHandler:
                    retryCounter = 0
                    while not self._isConnected() and (retryCounter < self.maxRetries + 2):
                        retryCounter += 1

                        # Try to login
                        try:
                            self._issueToken = self._gqlHandler.login()
                        except Exception:
                            pass

                        # Wait for next try
                        if not self._issueToken:
                            self.delay(self.retryDelay)
                    if self._issueToken:
                        self.dtIssueToken = int(time())
                else:
                    raise Exception("Unknown error creating a new GraphQL handler instance.")
            except Exception:
                # Login failed.
                self._logoff()
        return self._issueToken

    def init(self, forceLogin: bool = False) -> None:
        token = self.__login(forceLogin)
        if token:
            if token.get("user", {}):
                if not self._childPhoneNumber:
                    self.watchs = token.get("user", {}).get("children", _LIST_DICT)
                else:
                    for watch in token.get("user", {}).get("children", _LIST_DICT):
                        if watch["ward"]["phoneNumber"] in self._childPhoneNumber:
                            self.watchs.append(watch)
                self.user = token.get("user", {})
                return
        raise LoginError("Login to Xplora® API failed. Check your input!")

    def version(self) -> str:
        return "{0}-{1}".format(VERSION, VERSION_APP)

    def setDevices(self) -> List[Dict[str, Any]]:
        return self._setDevices()

    def _setDevices(self) -> List[str]:
        wuids: List[str] = self.getWatchUserIDs()
        for wuid in wuids:
            self.device[wuid] = {}
            self.device[wuid]["getWatchAlarm"] = self.getWatchAlarm(wuid=wuid)
            self.device[wuid]["loadWatchLocation"] = self.loadWatchLocation(wuid=wuid)
            self.device[wuid]["watch_battery"] = int(self.device[wuid]["loadWatchLocation"].get("watch_battery", -1))
            self.device[wuid]["watch_charging"] = self.device[wuid]["loadWatchLocation"].get("watch_charging", False)
            self.device[wuid]["locateType"] = self.device[wuid]["loadWatchLocation"].get(
                "locateType", LocationType.UNKNOWN.value
            )
            self.device[wuid]["isInSafeZone"] = self.device[wuid]["loadWatchLocation"].get("isInSafeZone", False)
            self.device[wuid]["safeZoneLabel"] = self.device[wuid]["loadWatchLocation"].get("safeZoneLabel", "")
            self.device[wuid]["getWatchSafeZones"] = self.getWatchSafeZones(wuid=wuid)
            self.device[wuid]["getSilentTime"] = self.getSilentTime(wuid=wuid)
            self.device[wuid]["getWatches"] = self.getWatches(wuid=wuid)
            self.device[wuid]["getSWInfo"] = self.getSWInfo(wuid=wuid, watches=self.device[wuid]["getWatches"])
            self.device[wuid]["getWatchState"] = self.getWatchState(wuid=wuid, watches=self.device[wuid]["getWatches"])
            d = datetime.now()
            dt = datetime(year=d.year, month=d.month, day=d.day)
            self.device[wuid]["getWatchUserSteps"] = self.getWatchUserSteps(wuid=wuid, date=dt.timestamp())
            self.device[wuid]["getWatchOnlineStatus"] = self.getWatchOnlineStatus(wuid=wuid)
            self.device[wuid]["getWatchUserIcons"] = self.getWatchUserIcons(wuid=wuid)
            self.device[wuid]["getWatchUserXcoins"] = self.getWatchUserXcoins(wuid=wuid)
        return wuids

    ##### Contact Info #####
    def getWatchUserContacts(self, wuid: str) -> List[Dict[str, Any]]:
        retryCounter = 0
        dataOk: List[Dict[str, Any]] = []
        contacts_raw: Dict[str, Any] = {}
        contacts: List[Dict[str, Any]] = []
        while not dataOk and (retryCounter < self.maxRetries + 2):
            retryCounter += 1
            try:
                contacts_raw = self._gqlHandler.getWatchUserContacts(wuid)
                _contacts: Dict[str, Any] = contacts_raw.get("contacts", {})
                if not _contacts:
                    dataOk.append({})
                    return contacts
                _contacts_contacts = _contacts.get("contacts", _LIST_DICT)
                if not _contacts_contacts:
                    dataOk.append({})
                    return contacts
                for contact in _contacts.get("contacts", _LIST_DICT):
                    try:
                        xcoin = contact["contactUser"]["xcoin"]
                        id = contact["contactUser"]["id"]
                    except TypeError:
                        # None - XCoins
                        xcoin = -1
                        id = None
                    contacts.append(
                        {
                            "id": id,
                            "guardianType": contact["guardianType"],
                            "create": datetime.fromtimestamp(contact["create"]).strftime("%Y-%m-%d %H:%M:%S"),
                            "update": datetime.fromtimestamp(contact["update"]).strftime("%Y-%m-%d %H:%M:%S"),
                            "name": contact["name"],
                            "phoneNumber": f"+{contact['countryPhoneNumber']}{contact['phoneNumber']}",
                            "xcoin": xcoin,
                        }
                    )
            except Exception as error:
                _LOGGER.debug(error)
            dataOk = contacts
            if not dataOk:
                self._logoff()
                self.delay(self.retryDelay)
        return contacts

    def getWatchAlarm(self, wuid: str) -> List[Dict[str, Any]]:
        retryCounter = 0
        dataOk: List[Dict[str, Any]] = []
        alarms_raw: Dict[str, Any] = {}
        alarms: List[Dict[str, Any]] = []
        while not dataOk and (retryCounter < self.maxRetries + 2):
            retryCounter += 1
            try:
                alarms_raw = self._gqlHandler.getAlarmTime(wuid)
                _alarms = alarms_raw.get("alarms", [])
                if not _alarms:
                    dataOk.append({})
                    return alarms
                for alarm in _alarms:
                    alarms.append(
                        {
                            "id": alarm["id"],
                            "vendorId": alarm["vendorId"],
                            "name": alarm["name"],
                            "start": self._helperTime(alarm["occurMin"]),
                            "weekRepeat": alarm["weekRepeat"],
                            "status": alarm["status"],
                        }
                    )
            except Exception as error:
                _LOGGER.debug(error)
            dataOk = alarms
            if not dataOk:
                self._logoff()
                self.delay(self.retryDelay)
        return alarms

    def loadWatchLocation(self, wuid: str = "", withAsk: bool = True) -> Dict[str, Any]:
        retryCounter = 0
        dataOk: List[Dict[str, Any]] = []
        location_raw: Dict[str, Any] = {}
        watch_location: Dict[str, Any] = {}
        while not dataOk and (retryCounter < self.maxRetries + 2):
            retryCounter += 1
            try:
                if withAsk:
                    self.askWatchLocate(wuid)
                location_raw = self._gqlHandler.getWatchLastLocation(wuid)
                _watchLastLocate = location_raw.get(
                    "watchLastLocate",
                    {},
                )
                if not _watchLastLocate:
                    dataOk = {}
                    return watch_location
                _tm = 31532399 if _watchLastLocate.get("tm") is None else _watchLastLocate.get("tm")
                _lat = "0.0" if _watchLastLocate.get("lat") is None else _watchLastLocate.get("lat")
                _lng = "0.0" if _watchLastLocate.get("lng") is None else _watchLastLocate.get("lng")
                _rad = -1 if _watchLastLocate.get("rad") is None else _watchLastLocate.get("rad")
                _poi = "" if _watchLastLocate.get("poi") is None else _watchLastLocate.get("poi")
                _city = "" if _watchLastLocate.get("city") is None else _watchLastLocate.get("city")
                _province = "" if _watchLastLocate.get("province") is None else _watchLastLocate.get("province")
                _country = "" if _watchLastLocate.get("country") is None else _watchLastLocate.get("country")
                _locateType = (
                    LocationType.UNKNOWN.value
                    if _watchLastLocate.get("locateType") is None
                    else _watchLastLocate.get("locateType")
                )
                _isInSafeZone = False if _watchLastLocate.get("isInSafeZone") is None else _watchLastLocate.get("isInSafeZone")
                _safeZoneLabel = "" if _watchLastLocate.get("safeZoneLabel") is None else _watchLastLocate.get("safeZoneLabel")
                _watch_battery = -1 if _watchLastLocate.get("battery") is None else _watchLastLocate.get("battery")
                _watch_charging = False if _watchLastLocate.get("isCharging") is None else _watchLastLocate.get("isCharging")
                watch_location = {
                    "tm": datetime.fromtimestamp(_tm).strftime("%Y-%m-%d %H:%M:%S"),
                    "lat": _lat,
                    "lng": _lng,
                    "rad": _rad,
                    "poi": _poi,
                    "city": _city,
                    "province": _province,
                    "country": _country,
                    "locateType": _locateType,
                    "isInSafeZone": _isInSafeZone,
                    "safeZoneLabel": _safeZoneLabel,
                    "watch_battery": _watch_battery,
                    "watch_charging": _watch_charging,
                    "watch_last_location": _watchLastLocate,
                }
            except Exception as error:
                _LOGGER.debug(error)
            dataOk = watch_location
            if not dataOk:
                self._logoff()
                self.delay(self.retryDelay)
        return watch_location

    def getWatchBattery(self, wuid: str) -> int:
        watch_b: Dict[str, Any] = self.loadWatchLocation(wuid=wuid)
        return int(watch_b.get("watch_battery", -1))

    def getWatchIsCharging(self, wuid: str) -> bool:
        watch_c: Dict[str, Any] = self.loadWatchLocation(wuid=wuid)
        if watch_c.get("watch_charging", False):
            return True
        return False

    def getWatchOnlineStatus(self, wuid: str) -> str:
        retryCounter = 0
        dataOk = WatchOnlineStatus.UNKNOWN
        asktrack_raw: WatchOnlineStatus = WatchOnlineStatus.UNKNOWN
        while dataOk is WatchOnlineStatus.UNKNOWN and (retryCounter < self.maxRetries + 2):
            retryCounter += 1
            try:
                ask_raw = self.askWatchLocate(wuid)
                track_raw = self.getTrackWatchInterval(wuid)
                if ask_raw or (track_raw != -1):
                    asktrack_raw = WatchOnlineStatus.ONLINE
                else:
                    asktrack_raw = WatchOnlineStatus.OFFLINE
            except Exception as error:
                _LOGGER.debug(error)
            dataOk = asktrack_raw
            if dataOk is WatchOnlineStatus.UNKNOWN:
                self._logoff()
                self.delay(self.retryDelay)
        return asktrack_raw.value

    """def __setReadChatMsg(self, msgId, id):
        return (self._gqlHandler.setReadChatMsg(self.getWatchUserIDs(), msgId, id))["setReadChatMsg"]"""

    def getWatchUnReadChatMsgCount(self, wuid: str) -> int:
        # bug?
        return (self._gqlHandler.unReadChatMsgCount(wuid)).get("unReadChatMsgCount", -1)

    def getWatchChats(self, wuid: str, offset: int = 0, limit: int = 100, msgId: str = "") -> List[Dict[str, Any]]:
        # bug?
        retryCounter = 0
        dataOk: List[Dict[str, Any]] = []
        chats_raw: Dict[str, Any] = {}
        chats: List[Dict[str, Any]] = []
        while not dataOk and (retryCounter < self.maxRetries + 2):
            retryCounter += 1
            try:
                chats_raw = self._gqlHandler.chats(wuid, offset, limit, msgId)
                _chatsNew = chats_raw.get("chatsNew", {})
                if not _chatsNew:
                    dataOk.append({})
                    return chats
                _list = _chatsNew.get("list", [])
                if not _list:
                    dataOk.append({})
                    return chats
                for chat in chats_raw["chatsNew"]["list"]:
                    chats.append(
                        {
                            "msgId": chat["msgId"],
                            "type": chat["type"],
                            # chat['sender'],
                            "sender_id": chat["sender"]["id"],
                            "sender_name": chat["sender"]["name"],
                            # chat['receiver'],
                            "receiver_id": chat["receiver"]["id"],
                            "receiver_name": chat["receiver"]["name"],
                            # chat['data'],
                            "data_text": chat["data"]["text"],
                            "data_sender_name": chat["data"]["sender_name"],
                            "create": datetime.fromtimestamp(chat["create"]).strftime("%Y-%m-%d %H:%M:%S"),
                        }
                    )
            except Exception as error:
                _LOGGER.debug(error)
            dataOk = chats
            if not dataOk:
                self._logoff()
                self.delay(self.retryDelay)
        return chats

    ##### Watch Location Info #####
    def getWatchLastLocation(self, wuid: str, withAsk: bool = False) -> Dict[str, Any]:
        _loadWatchLocation = self.loadWatchLocation(wuid=wuid, withAsk=withAsk)
        if isinstance(_loadWatchLocation, dict):
            return _loadWatchLocation.get("watch_last_location", {}) 
        if not _loadWatchLocation:
            return {}
        for loadWatchLocation in _loadWatchLocation:
            return loadWatchLocation.get("watch_last_location", {})
        return {}

    def getWatchLocate(self, wuid: str) -> Dict[str, Any]:
        _loadWatchLocation = self.loadWatchLocation(wuid=wuid)
        if isinstance(_loadWatchLocation, dict):
            return _loadWatchLocation
        if not _loadWatchLocation:
            return {}
        for loadWatchLocation in _loadWatchLocation:
            return loadWatchLocation
        return {}

    def getWatchLocateType(self, wuid: str) -> str:
        return self.getWatchLocate(wuid).get("locateType", LocationType.UNKNOWN.value)

    def getWatchIsInSafeZone(self, wuid: str) -> bool:
        return self.getWatchLocate(wuid).get("isInSafeZone", False)

    def getWatchSafeZoneLabel(self, wuid: str) -> str:
        return self.getWatchLocate(wuid).get("safeZoneLabel", "")

    def getWatchSafeZones(self, wuid: str) -> List[Dict[str, Any]]:
        retryCounter = 0
        dataOk: List[Dict[str, Any]] = []
        safeZones_raw: Dict[str, Any] = {}
        safe_zones: List[Dict[str, Any]] = []
        while not dataOk and (retryCounter < self.maxRetries + 2):
            retryCounter += 1
            try:
                safeZones_raw = self._gqlHandler.safeZones(wuid)
                _safeZones = safeZones_raw.get("safeZones", [])
                if not _safeZones:
                    dataOk.append({})
                    return safe_zones
                for safeZone in _safeZones:
                    safe_zones.append(
                        {
                            "vendorId": safeZone["vendorId"],
                            "groupName": safeZone["groupName"],
                            "name": safeZone["name"],
                            "lat": safeZone["lat"],
                            "lng": safeZone["lng"],
                            "rad": safeZone["rad"],
                            "address": safeZone["address"],
                        }
                    )
            except Exception as error:
                _LOGGER.debug(error)
            dataOk = safe_zones
            if not dataOk:
                self._logoff()
                self.delay(self.retryDelay)
        return safe_zones

    def getTrackWatchInterval(self, wuid: str) -> int:
        return self._gqlHandler.trackWatch(wuid).get("trackWatch", -1)

    def askWatchLocate(self, wuid: str) -> bool:
        return self._gqlHandler.askWatchLocate(wuid).get("askWatchLocate", False)

    ##### Feature #####
    def getSilentTime(self, wuid: str) -> List[Dict[str, Any]]:
        retryCounter = 0
        dataOk: List[Dict[str, Any]] = []
        silentTimes_raw: Dict[str, Any] = {}
        school_silent_mode: List[Dict[str, Any]] = []
        while not dataOk and (retryCounter < self.maxRetries + 2):
            retryCounter += 1
            try:
                silentTimes_raw = self._gqlHandler.silentTimes(wuid)
                _silentTimes = silentTimes_raw.get("silentTimes", [])
                if not _silentTimes:
                    dataOk.append({})
                    return school_silent_mode
                for silentTime in _silentTimes:
                    school_silent_mode.append(
                        {
                            "id": silentTime["id"],
                            "vendorId": silentTime["vendorId"],
                            "start": self._helperTime(silentTime["start"]),
                            "end": self._helperTime(silentTime["end"]),
                            "weekRepeat": silentTime["weekRepeat"],
                            "status": silentTime["status"],
                        }
                    )
            except Exception as error:
                _LOGGER.debug(error)
            dataOk = school_silent_mode
            if not dataOk:
                self._logoff()
                self.delay(self.retryDelay)
        return school_silent_mode

    def setEnableSilentTime(self, silentId: str, wuid: str) -> bool:
        retryCounter = 0
        dataOk = ""
        _raw = ""
        while not dataOk and (retryCounter < self.maxRetries + 2):
            retryCounter += 1
            try:
                enable_raw = self._gqlHandler.setEnableSlientTime(silentId)
                _setEnableSilentTime = enable_raw.get("setEnableSilentTime", -1)
                if not _setEnableSilentTime:
                    dataOk = "0"
                    return bool(_raw)
                _raw = _setEnableSilentTime
            except Exception as error:
                _LOGGER.debug(error)
            dataOk = _raw
            if not dataOk:
                self._logoff()
                self.delay(self.retryDelay)
        return bool(_raw)

    def setDisableSilentTime(self, silentId: str, wuid: str) -> bool:
        retryCounter = 0
        dataOk = ""
        _raw = ""
        while not dataOk and (retryCounter < self.maxRetries + 2):
            retryCounter += 1
            try:
                disable_raw = self._gqlHandler.setEnableSlientTime(silentId, NormalStatus.DISABLE.value)
                _setEnableSilentTime = disable_raw.get("setEnableSilentTime", -1)
                if not _setEnableSilentTime:
                    dataOk = "0"
                    return bool(_raw)
                _raw = _setEnableSilentTime
            except Exception as error:
                _LOGGER.debug(error)
            dataOk = _raw
            if not dataOk:
                self._logoff()
                self.delay(self.retryDelay)
        return bool(_raw)

    def setAllEnableSilentTime(self, wuid: str) -> List[bool]:
        res: List[bool] = []
        for silentTime in self.getSilentTime(wuid):
            res.append(self.setEnableSilentTime(silentTime.get("id", ""), wuid))
        return res

    def setAllDisableSilentTime(self, wuid: str) -> List[bool]:
        res: List[bool] = []
        for silentTime in self.getSilentTime(wuid):
            res.append(self.setDisableSilentTime(silentTime.get("id", ""), wuid))
        return res

    def setEnableAlarmTime(self, alarmId: str, wuid: str) -> bool:
        retryCounter = 0
        dataOk = ""
        _raw = ""
        while not dataOk and (retryCounter < self.maxRetries + 2):
            retryCounter += 1
            try:
                enable_raw = self._gqlHandler.setEnableAlarmTime(alarmId)
                _modifyAlarm = enable_raw.get("modifyAlarm", -1)
                if not _modifyAlarm:
                    dataOk = "0"
                    return bool(_raw)
                _raw = _modifyAlarm
            except Exception as error:
                _LOGGER.debug(error)
            dataOk = _raw
            if not dataOk:
                self._logoff()
                self.delay(self.retryDelay)
        return bool(_raw)

    def setDisableAlarmTime(self, alarmId: str, wuid: str) -> bool:
        retryCounter = 0
        dataOk = ""
        _raw = ""
        while not dataOk and (retryCounter < self.maxRetries + 2):
            retryCounter += 1
            try:
                disable_raw = self._gqlHandler.setEnableAlarmTime(alarmId, NormalStatus.DISABLE.value)
                _modifyAlarm = disable_raw.get("modifyAlarm", -1)
                if not _modifyAlarm:
                    dataOk = "0"
                    return bool(_raw)
                _raw = _modifyAlarm
            except Exception as error:
                _LOGGER.debug(error)
            dataOk = _raw
            if not dataOk:
                self._logoff()
                self.delay(self.retryDelay)
        return bool(_raw)

    def setAllEnableAlarmTime(self, wuid: str) -> List[bool]:
        res: List[bool] = []
        for alarmTime in self.getWatchAlarm(wuid):
            res.append(self.setEnableAlarmTime(alarmTime.get("id", ""), wuid))
        return res

    def setAllDisableAlarmTime(self, wuid: str) -> List[bool]:
        res: List[bool] = []
        for alarmTime in self.getWatchAlarm(wuid):
            res.append(self.setDisableAlarmTime(alarmTime.get("id", ""), wuid))
        return res

    def sendText(self, text: str, wuid: str) -> bool:
        # sender is login User
        return self._gqlHandler.sendText(wuid, text)

    def isAdmin(self, wuid: str) -> bool:
        contacts = self.getWatchUserContacts(wuid)
        for contact in contacts:
            _id = contact.get("id", None)
            if self.getUserID() == _id:
                if contact["guardianType"] == "FIRST":
                    return True
        return False

    def shutdown(self, wuid: str) -> bool:
        if self.isAdmin(wuid):
            return self._gqlHandler.shutdown(wuid)
        raise NoAdminError()

    def reboot(self, wuid: str) -> bool:
        if self.isAdmin(wuid):
            return self._gqlHandler.reboot(wuid)
        raise NoAdminError()

    def getFollowRequestWatchCount(self) -> int:
        c: Dict[str, Any] = self._gqlHandler.getFollowRequestWatchCount()
        return c.get("followRequestWatchCount", 0)

    def getWatches(self, wuid: str) -> Dict[str, Any]:
        retryCounter = 0
        dataOk: Dict[str, Any] = {}
        watches_raw: Dict[str, Any] = {}
        watches: Dict[str, Any] = {}
        while not dataOk and (retryCounter < self.maxRetries + 2):
            retryCounter += 1
            try:
                watches_raw = self._gqlHandler.getWatches(wuid)
                _watches: List[Dict[str, Any]] = watches_raw.get("watches", [])
                if not _watches:
                    dataOk = {}
                    return watches
                for watch in _watches:
                    watches = {
                        "imei": watch["swKey"],
                        "osVersion": watch["osVersion"],
                        "qrCode": watch["qrCode"],
                        "model": watch["groupName"],
                    }
            except Exception as error:
                _LOGGER.debug(error)
            dataOk = watches
            if not dataOk:
                self._logoff()
                self.delay(self.retryDelay)
        return watches

    def getSWInfo(self, wuid: str, watches: Dict[str, Any] = {}) -> Dict[str, Any]:
        wqr: Dict[str, Any] = watches if watches else self.getWatches(wuid=wuid)
        qrCode: str = wqr.get("qrCode", "=")
        return self._gqlHandler.getSWInfo(qrCode.split("=")[1])

    def getWatchState(self, wuid: str, watches: Dict[str, Any] = {}) -> Dict[str, Any]:
        wqr: Dict[str, Any] = watches if watches else self.getWatches(wuid=wuid)
        qrCode: str = wqr.get("qrCode", "=")
        try:
            return self._gqlHandler.getWatchState(qrCode=qrCode.split("=")[1])
        except Exception as err:
            print(err)

    def conv360IDToO2OID(self, qid: str, deviceId: str) -> Dict[str, Any]:
        return self._gqlHandler.conv360IDToO2OID(qid, deviceId)

    def campaigns(self, id: str, categoryId: str) -> Dict[str, Any]:
        return self._gqlHandler.campaigns(id, categoryId)

    def getCountries(self) -> List[Dict[str, str]]:
        countries: Dict[str, Any] = self._gqlHandler.countries()
        return countries.get("countries", {})

    def getWatchLocHistory(self, wuid: str, date: int, tz: str, limit: int) -> Dict[str, Any]:
        return self._gqlHandler.getWatchLocHistory(wuid, date, tz, limit)

    def watchesDynamic(self) -> Dict[str, Any]:
        return self._gqlHandler.watchesDynamic()

    def watchGroups(self, id: str = "") -> Dict[str, Any]:
        return self._gqlHandler.watchGroups(id)

    def familyInfo(self, wuid: str, watchId: str, tz: str, date: int) -> Dict[str, Any]:
        return self._gqlHandler.familyInfo(wuid, watchId, tz, date)

    def avatars(self, id: str) -> Dict[str, Any]:
        return self._gqlHandler.avatars(id)

    def getWatchUserSteps(self, wuid: str, date: int) -> Dict[str, Any]:
        userSteps = self._gqlHandler.getWatchUserSteps(wuid=wuid, tz=self._timeZone, date=date)
        if not userSteps:
            return {}
        userSteps = userSteps.get("userSteps", {})
        if not userSteps:
            return {}
        return userSteps

    def addStep(self, step: int) -> bool:
        s: Dict[str, bool] = self._gqlHandler.addStep(step)
        return s.get("addStep", False)

    def submitIncorrectLocationData(self, wuid: str, lat: str, lng: str, timestamp: str) -> bool:
        data: Dict[str, bool] = self._gqlHandler.submitIncorrectLocationData(wuid, lat, lng, timestamp)
        return data.get("submitIncorrectLocationData", False)



from __future__ import annotations

import hashlib
import math
from time import time
from datetime import datetime, timezone
from typing import Any, Dict

from .const import API_KEY, API_SECRET


class HandlerGQL:
    def __init__(
        self,
        countryPhoneNumber: str,
        phoneNumber: str,
        password: str,
        userLang: str,
        timeZone: str,
    ) -> None:
        # init vars
        self.sessionId = None
        self.accessToken = None
        self.accessTokenExpire = 0
        self.userLocale = userLang
        self.timeZone = timeZone
        self.countryPhoneNumber = countryPhoneNumber
        self.phoneNumber = phoneNumber
        self.passwordMD5 = hashlib.md5(password.encode()).hexdigest()
        self._API_KEY = API_KEY
        self._API_SECRET = API_SECRET
        self.issueDate = 0
        self.expireDate = 0
        self.userId = None
        self.variables = {
            "countryPhoneNumber": self.countryPhoneNumber,
            "phoneNumber": self.phoneNumber,
            "password": self.passwordMD5,
            "userLang": self.userLocale,
            "timeZone": self.timeZone,
        }
        self.issueToken: Dict[str, Any]

    def c(self) -> HandlerGQL:
        return self

    def getRequestHeaders(self, acceptedContentType: str) -> Dict[str, Any]:
        if acceptedContentType == "" or acceptedContentType is None:
            raise Exception("acceptedContentType MUST NOT be empty!")
        if self._API_KEY is None:
            raise Exception("Xplorao2o API_KEY MUST NOT be empty!")
        if self._API_SECRET is None:
            raise Exception("Xplorao2o API_SECRET MUST NOT be empty!")
        requestHeaders = {}
        if self.accessToken is None:
            # OPEN authorization
            authorizationHeader = f"Open {self._API_KEY}:{self._API_SECRET}"
        else:
            # BEARER authorization
            authorizationHeader = f"Bearer {self.accessToken}:{self._API_SECRET}"
            rfc1123DateString = datetime.now(timezone.utc).strftime("%a, %d %b %Y %H:%M:%S") + " GMT"
            requestHeaders["H-Date"] = rfc1123DateString
            requestHeaders["H-Authorization"] = authorizationHeader
        requestHeaders["H-BackDoor-Authorization"] = authorizationHeader
        requestHeaders["Accept"] = acceptedContentType
        requestHeaders["Content-Type"] = acceptedContentType
        requestHeaders["H-Tid"] = str(math.floor(time()))
        return requestHeaders
